/* -*- c++ -*-
 *
 *  Le Caviste --- a wine cellar managing application
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  For more information, visit the Le Caviste Home Page at
 *  http://www.lecaviste.org/
 *
 *  Copyright (c) 2007
 *      Le Caviste Authors <devel@lecaviste.org>
 *
 */
#include "appellation_dialog.hpp"
#include "cellar.hpp"

AppellationDialog::AppellationDialog (
	Cellar * cellar, 
	QWidget * parent
) : QDialog(parent), 
	m_cellar(cellar), m_appellationIdx(-1)
{
  m_model = cellar->appellations();
  createUI();
  setWindowTitle(tr("Add an appellation"));
}

AppellationDialog::AppellationDialog (
	Cellar * cellar, 
	int id,
	QWidget * parent
) : QDialog(parent), 
	m_cellar(cellar), m_appellationIdx(id)
{
  m_model = cellar->appellations();

  createUI();
  
#ifdef DEBUG
  m_groupBox->setTitle(tr("Edit an appellation (%1)").arg(id));
#else
  m_groupBox->setTitle(tr("Edit an appellation"));
#endif
  setWindowTitle(tr("Edit an appellation"));
  fillAppellation();
}

void AppellationDialog::createUI(void)
{
  m_groupBox = new QGroupBox(tr("Add an appellation"));

  QLabel * nameLabel = new QLabel(tr("Name:"));
  m_name = new QLineEdit;
  QLabel * countryLabel = new QLabel(tr("Country:"));
  m_country = new QComboBox;
  m_cellar->countries()->setFilter("");
  m_country->setModel(m_cellar->countries());
  m_country->setModelColumn(1);
  m_country->model()->sort(1);
  m_country->setCurrentIndex(0);

  QGridLayout * boxLayout = new QGridLayout;
  int idx = 0;
  boxLayout->addWidget(nameLabel, idx, 0, Qt::AlignTop);
  boxLayout->addWidget(m_name, idx++, 1);
  boxLayout->addWidget(countryLabel, idx, 0, Qt::AlignTop);
  boxLayout->addWidget(m_country, idx++, 1);
  m_groupBox->setLayout(boxLayout);

  // Create buttons
  QPushButton * closeButton = new QPushButton(tr("&Close"));
  QPushButton * validateButton = new QPushButton(tr("&Ok"));
  
  connect(closeButton, SIGNAL(clicked()), this, SLOT(close()));
  connect(validateButton, SIGNAL(clicked()), this, SLOT(submit()));
  
  QDialogButtonBox * buttonBox = new QDialogButtonBox;
  buttonBox->addButton(validateButton, QDialogButtonBox::ApplyRole);
  buttonBox->addButton(closeButton, QDialogButtonBox::RejectRole);
  
  // Create main layout
  QVBoxLayout * layout = new QVBoxLayout;
  layout->addWidget(m_groupBox);
  layout->addWidget(buttonBox);
  setLayout(layout);
}

void AppellationDialog::fillAppellation(void)
{
  QSqlQuery query;
  // Get selected appellation infos
  QSqlRecord record = m_model->record(m_appellationIdx);
  m_name->setText(record.value(Cellar::AppellationColumn_Name).toString());
  int pos = m_country->findText(
	record.value(Cellar::AppellationColumn_Country).toString());
  m_country->setCurrentIndex(pos);
}


void AppellationDialog::submit(void)
{
  QString name = m_name->text();
  
  if (name.isEmpty()) {
	  QString message(tr("Please provide the name of the appellation."));
	  QMessageBox::information(this, tr("Add an appellation"), message);
  } else {
	QSqlRecord record;
	
	QSqlField f1("name", QVariant::String);
	QSqlField f2("country_id", QVariant::Int);
	
	f1.setValue(QVariant(name));
	int country = m_cellar->findCountryId(m_country->currentText());
	f2.setValue(QVariant(country));
    record.append(f1);
    record.append(f2);

	if (m_appellationIdx == -1) {
	  if (!m_model->insertRecord(-1, record))
		qDebug() << "Error inserting " << name << " (" << country << "): "
				 << m_model->database().lastError().text();
	} else {
	  m_model->setRecord(m_appellationIdx, record);
	}

	m_appellation = name;
	m_model->submitAll();
	accept();
  }
}
