/* -*- c++ -*-
 *
 *  Le Caviste --- a wine cellar managing application
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  For more information, visit the Le Caviste Home Page at
 *  http://www.lecaviste.org/
 *
 *  Copyright (c) 2007
 *      Le Caviste Authors <devel@lecaviste.org>
 *
 */
#include "appellations_dialog.hpp"
#include "appellation_dialog.hpp"

AppellationsDialog::AppellationsDialog(
   Cellar * cellar, 
   QWidget * parent
) : QDialog(parent), m_cellar(cellar)
{
  m_model = cellar->appellations();
  m_model->setFilter("appellation.id>0");

  m_view = new QTableView;
  m_view->setModel(m_model);

  m_view->setColumnHidden(Cellar::AppellationColumn_Id, true);
  m_view->setSortingEnabled(false);
  m_view->setAlternatingRowColors(true);
  m_view->setSelectionMode(QAbstractItemView::SingleSelection);
  m_view->setSelectionBehavior(QAbstractItemView::SelectRows);
  m_view->resizeColumnsToContents();
  m_view->setTextElideMode(Qt::ElideRight);
  m_view->setCurrentIndex(QModelIndex());
  m_view->sortByColumn(Cellar::AppellationColumn_Name, Qt::AscendingOrder);

  connect(m_view->selectionModel(), 
		  SIGNAL(currentRowChanged(const QModelIndex &, const QModelIndex &)), 
		  this, SLOT(onCurrentRowChanged(const QModelIndex&)));
  connect(m_view, SIGNAL(doubleClicked(const QModelIndex &)),
		  this, SLOT(onItemDoubleClicked(const QModelIndex &)));

  QHeaderView * header = m_view->horizontalHeader();
  header->setClickable(false);
  header->setAlternatingRowColors(true);
  header->setDefaultAlignment(Qt::AlignBottom);

  header = m_view->verticalHeader();
  header->hide();
  header->setDefaultAlignment(Qt::AlignBottom);

  m_toolbar = new QToolBar;

  m_actionAdd = new QAction(this);
  m_actionAdd->setIcon(
	QIcon(QString::fromUtf8(":/images/icons/vcard_add.png")));
  m_actionAdd->setText(tr("Add"));
  m_actionAdd->setStatusTip(tr("Add a new appellation."));
  connect(m_actionAdd, SIGNAL(triggered()), 
		  this, SLOT(onActionAdd()));
  m_toolbar->addAction(m_actionAdd);

  m_actionEdit = new QAction(this);
  m_actionEdit->setIcon(
	QIcon(QString::fromUtf8(":/images/icons/vcard_edit.png")));
  m_actionEdit->setText(tr("Edit"));
  m_actionEdit->setStatusTip(tr("Edit selected appellation."));
  m_actionEdit->setEnabled(false);
  connect(m_actionEdit, SIGNAL(triggered()), 
		  this, SLOT(onActionEdit()));
  m_toolbar->addAction(m_actionEdit);

  m_actionDelete = new QAction(this);
  m_actionDelete->setText(tr("Delete"));
  m_actionDelete->setStatusTip(tr("Delete selected appellation."));
  m_actionDelete->setIcon(
	QIcon(QString::fromUtf8(":/images/icons/vcard_delete.png")));
  m_actionDelete->setEnabled(false);

  connect(m_actionDelete, SIGNAL(triggered()), 
		  this, SLOT(onActionDelete()));
  m_toolbar->addAction(m_actionDelete);

  m_actionClose = new QAction(this);
  m_actionClose->setText(tr("Close"));
  m_actionClose->setStatusTip(tr("Close window"));
  m_actionClose->setIcon(
	QIcon(QString::fromUtf8(":/images/icons/cancel.png")));

  connect(m_actionClose, SIGNAL(triggered()), 
		  this, SLOT(close()));
  m_toolbar->addSeparator();
  m_toolbar->addAction(m_actionClose);


  // Create main layout
  QVBoxLayout * layout = new QVBoxLayout;
  layout->addWidget(m_toolbar);
  layout->addWidget(m_view);
  setLayout(layout);
  
  setWindowTitle(tr("Appellations"));
  readSettings();
}


void AppellationsDialog::onActionDelete(void)
{
  QSqlQuery query;
  QModelIndex index = m_view->currentIndex();
  if (!index.isValid()) return;

  // Get selected appellation infos
  QSqlRecord record = m_model->record(index.row());
  int id = record.value("id").toInt();
  QString name = record.value("name").toString();

  // Get the number of wines affected to this appellation
  query.exec(QString("SELECT COUNT(*) FROM wine WHERE appellation_id=%1")
			 .arg(id));
  int wines = 0;
  if (query.next())
	wines = query.value(0).toInt();
  
  int r;

  if (wines > 0) {
	r = QMessageBox::question(
		  this, tr("Delete appellation"),
		  tr("Appellation \"%1\" is associated with %2 wines.\nDelete it?")
		  .arg(name).arg(wines),
		  QMessageBox::Yes | QMessageBox::Default,
		  QMessageBox::No | QMessageBox::Escape);
  } else {
	r = QMessageBox::question(this, tr("Delete appellation"),
							  tr("Delete appellation \"%1\"?").arg(name),
							  QMessageBox::Yes | QMessageBox::Default,
							  QMessageBox::No | QMessageBox::Escape);
  }

  if (r == QMessageBox::No) {
	return;
  }

  // Update wines associated with removed appellation
  if (wines > 0) {
	query.exec(QString("UPDATE wine SET appellation_id=0 WHERE appellation_id=%1")
			   .arg(id));
  }
  m_model->removeRow(index.row());
  m_model->submitAll();
}

void AppellationsDialog::onActionAdd(void)
{
  AppellationDialog * dialog = 
	new AppellationDialog(m_cellar, this);
  if (dialog->exec()) {
	QString el = dialog->appellation();
	for(int i=0; i<m_model->rowCount(); i++) {
	  QSqlRecord record = m_model->record(i);
 	  QString name = record.value("name").toString();
 	  if (name == el) {
		m_view->setCurrentIndex(m_model->index(i,0));
 		m_view->scrollTo(m_model->index(i,0));
 		return;
 	  }
	}	
  }
}

void AppellationsDialog::onActionEdit(void)
{
  QModelIndex index = m_view->currentIndex();
  if (!index.isValid()) return;

  AppellationDialog * dialog = 
	new AppellationDialog(m_cellar, index.row(), this);
  dialog->exec();  
  m_view->setCurrentIndex(index);
  m_view->scrollTo(index);
}

void AppellationsDialog::onCurrentRowChanged(const QModelIndex& index)
{
  if (index.isValid()) {
	m_actionEdit->setEnabled(true);
	m_actionDelete->setEnabled(true);
  } else {
	m_actionEdit->setEnabled(false);
	m_actionDelete->setEnabled(false);
  }
}

void AppellationsDialog::onItemDoubleClicked(const QModelIndex& index)
{
  int row = index.row();
  AppellationDialog * dialog = 
	new AppellationDialog(m_cellar, index.row(), this);
  dialog->exec();  
  m_view->setCurrentIndex(m_model->index(row, 0));
}

void AppellationsDialog::closeEvent(QCloseEvent * event)
{
  writeSettings();
  event->accept();
}

void AppellationsDialog::writeSettings(void)
{
  QSettings settings("lecaviste.org", "Le Caviste");
  settings.beginGroup("AppellationsDialog");

  settings.setValue("size", geometry().size());

  for (int i=0; i<=ColumnsCount; i++) {
	if (!m_view->isColumnHidden(i)) {
	  settings.setValue(QString("col%1").arg(i,2, 10, QChar('0')), 
						m_view->columnWidth(i));
	}
  }
  settings.endGroup();
}


void AppellationsDialog::readSettings(void)
{
  QSettings settings("lecaviste.org", "Le Caviste");

  settings.beginGroup("AppellationsDialog");

  QSize size = settings.value("size", QSize(600, 400)).toSize();
  resize(size);

  for (int i=0; i<=ColumnsCount; i++) {
	int w = settings.value(QString("col%1").arg(i,2, 10, QChar('0')), 100)
	  .toInt();
	m_view->setColumnWidth(i, w);
  }
  settings.endGroup();
}
