/* -*- c++ -*-
 *
 *  Le Caviste --- a wine cellar managing application
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  For more information, visit the Le Caviste Home Page at
 *  http://www.lecaviste.org/
 *
 *  Copyright (c) 2007
 *      Le Caviste Authors <devel@lecaviste.org>
 *
 */
#include <QtXml>

#include "cellar.hpp"
#include "csv_parser.hpp"

CellarStats::CellarStats(void) :
  m_wines(-1), m_bottles(-1)
{
}


CellarStats::CellarStats(int wines, int bottles) :
  m_wines(wines), m_bottles(bottles)
{
}


Cellar::Cellar(QObject *) : 
  m_fileName(""), m_isOpened(false)
{
  m_db = QSqlDatabase::addDatabase("QSQLITE");
}

bool Cellar::open(const QString & fileName)
{
  m_db.setDatabaseName(fileName);
  if (!m_db.open())	{
	return false;
  }
  
  // Check if file is correct
  if (!check())	{
	m_db.close();
	// Try to convert
	if (!convertFromXml(fileName))
	  return false;
  } 

  createSqlModel();

  m_isOpened = true;
  m_fileName = fileName;
  return true;
}


bool Cellar::create(const QString & fileName)
{
  QString sqlQuery;

  m_db.setDatabaseName(fileName);
  if (!m_db.open()) {
	qDebug() << "Unable to establish a database connection.";
	return false;
  }
  
  QSqlQuery query(m_db);
  
  if (!query.exec ("CREATE TABLE wine ("
				   ""  "id INTEGER PRIMARY KEY AUTOINCREMENT,"
				   ""  "name TEXT NOT NULL,"
				   ""  "category_id INTEGER NOT NULL,"
				   ""  "appellation_id INTEGER NOT NULL,"
				   ""  "millesime INTEGER,"
				   ""  "volume INTEGER NOT NULL,"
				   ""  "quantity INTEGER NOT NULL,"
				   ""  "country_id INTEGER NOT NULL,"
				   ""  "productor_id INTEGER NOT NULL,"
				   ""  "place_id INTEGER,"
				   ""  "bought_date TEXT,"
				   ""  "bought_quantity INTEGER,"
				   ""  "bought_price REAL,"
				   ""  "best_before INTEGER,"
				   ""  "comment TEXT"
				   ")")) {
	qDebug() << "Error creating table wine: " 
			 << query.lastError();
	return false;
  }
  
  
  if (!query.exec ("CREATE TABLE appellation ("
				   ""  "id INTEGER PRIMARY KEY AUTOINCREMENT,"
				   ""  "name TEXT NOT NULL,"
				   ""  "country_id INTEGER NOT NULL"
				   ")")) {
	qDebug() << "Error creating table appellation: " 
			 << query.lastError();
	return false;
  }
  query.exec ("INSERT INTO appellation VALUES (0, \"" + tr("-- Unknown --") + "\", 0)");


  if (!query.exec ("CREATE TABLE country ("
				   ""  "id INTEGER PRIMARY KEY AUTOINCREMENT,"
				   ""  "name TEXT NOT NULL"
				   ")")) {
	qDebug() << "Error creating table country: " 
			 << query.lastError();
	return false;
  }
  query.exec ("INSERT INTO country VALUES (0, \"" + tr("-- Unknown --") + "\")");

  if (!query.exec ("CREATE TABLE productor ("
				   ""  "id INTEGER PRIMARY KEY AUTOINCREMENT,"
				   ""  "name TEXT NOT NULL,"
				   ""  "address TEXT,"
				   ""  "email TEXT,"
				   ""  "phone TEXT,"
				   ""  "fax TEXT"
				   ")")) {
	qDebug() << "Error creating table productor: " 
			 << query.lastError();
	return false;
  }
  sqlQuery = "INSERT INTO productor(id, name) "
	"VALUES (0, \"" + tr("-- Unknown --") + "\")";
  if (!query.exec (sqlQuery)) {
	qDebug() << "Error creating table productor: " 
			 << query.lastError();
  }

  if (!query.exec ("CREATE TABLE place ("
				   ""  "id INTEGER PRIMARY KEY AUTOINCREMENT,"
				   ""  "name TEXT NOT NULL"
				   ")")) {
	qDebug() << "Error creating table place: " 
			 << query.lastError();
	return false;
  }
  query.exec ("INSERT INTO place VALUES (0, \"" + tr("-- Unknown --") + "\")");


  if (!query.exec ("CREATE TABLE volume ("
				   ""  "id INTEGER PRIMARY KEY AUTOINCREMENT,"
				   ""  "name TEXT NOT NULL"
				   ")")) {
	qDebug() << "Error creating table volume: " 
			 << query.lastError();
	return false;
  }
  qDebug() << "Create volume table";
  for (int v=0; v<=Volume_Magnum; v++) {
	query.exec (QString("INSERT INTO volume VALUES (%1, \"%2\")")
				.arg(v).arg(volumeName((Volume)v)));
  }

  if (!query.exec ("CREATE TABLE category ("
				   ""  "id INTEGER PRIMARY KEY AUTOINCREMENT,"
				   ""  "name TEXT NOT NULL"
				   ")")) {
	qDebug() << "Error creating table category: " 
			 << query.lastError();
	return false;
  }
  qDebug() << "Create category table";
  for (int c=0; c<=Category_Liqueur; c++) {
	if (!query.exec (QString("INSERT INTO category VALUES (%1, \"%2\")")
					 .arg(c).arg(categoryName((Category)c)))) {
	  qDebug() << "Error inserting in table category: " 
			   << query.lastError();
	  return false;
	}
  }

  createSqlModel();

  m_isOpened = true;
  m_fileName = fileName;
  return true;
}


bool Cellar::close(void)
{
  m_isOpened = false;
  delete m_model;
  m_db.close();  
  return true;
}

bool Cellar::exportCSV(const QString & fileName)
{
  QFile file(fileName);
  if (!file.open(QIODevice::ReadWrite | QIODevice::Text)) {
	return false;
  }
  
  QTextStream out(&file);
  // Write header
  out << "\"" << tr("Name") << "\"" << ";";
  out << "\"" << tr("Category") << "\"" << ";";
  out << "\"" << tr("Appellation") << "\"" << ";";
  out << "\"" << tr("Millesime") << "\"" << ";";
  out << "\"" << tr("Volume") << "\"" << ";";
  out << "\"" << tr("Quantity") << "\"" << ";";
  out << "\"" << tr("Country") << "\"" << ";";
  out << "\"" << tr("Productor") << "\"" << ";";
  out << "\"" << tr("Place") << "\"" << ";";
  out << "\"" << tr("Bought Date") << "\"" << ";";
  out << "\"" << tr("Bought Quantity") << "\"" << ";";
  out << "\"" << tr("Bought Price") << "\"" << ";";
  out << "\"" << tr("Best Before") << "\"" << ";";
  out << "\"" << tr("Comment") << "\"" 	<< ";";
  out << endl;

  QSqlRecord record = m_model->record(0);
  int i = 0;
  while (!record.isNull("id")) {
	out << "\"" << record.value(WineColumn_Name).toString() << "\"" << ";";
	out << "\"" << record.value(WineColumn_Category).toString() << "\"" << ";";
	out << "\"" << record.value(WineColumn_Appellation).toString() << "\"" << ";";
	out << "\"" << record.value(WineColumn_Millesime).toString() << "\"" << ";";
	out << "\"" << record.value(WineColumn_Volume).toString() << "\"" << ";";
	out <<         record.value(WineColumn_Quantity).toInt() << ";";
	out << "\"" << record.value(WineColumn_Country).toString() << "\"" << ";";
	out << "\"" << record.value(WineColumn_Productor).toString() << "\"" << ";";
	out << "\"" << record.value(WineColumn_Place).toString() << "\"" << ";";
	out << "\"" << record.value(WineColumn_BoughtDate).toString() << "\"" << ";";
	out <<         record.value(WineColumn_BoughtQuantity).toInt() << ";";
	out <<         record.value(WineColumn_BoughtPrice).toDouble() << ";";
	out <<         record.value(WineColumn_BestBefore).toInt() << ";";
	out << "\"" << record.value(WineColumn_Comment).toString() << "\"" 	<< ";";
	out << endl;
	i++;
	record = m_model->record(i);
  }
  
  file.close();
  return true;
}

// Get the text from the CSV string list base on columnDefinition
QString Cellar::text(const QStringList & list,
					 const QHash<int, int> columnDefinition,
					 const WineColumn col)
{
  int pos = columnDefinition[col];
  if (pos == -1) return "";
  return list[pos];
}

int Cellar::findOrInsert(const QString & table, const QString & name)
{
  if (name.isEmpty()) return 0;

  QSqlQuery query;
  QString sqlQuery = QString("SELECT id "
							 "FROM %1 "
							 "WHERE name=%2").arg(table).arg(name);
  query.exec(sqlQuery);
  if (query.next()) {
	return query.value(0).toInt();
  } else {
	sqlQuery = QString("INSERT INTO %1 "
					   "(name) VALUES (\"%2\");").arg(table).arg(name);

	if (!query.exec(sqlQuery)) {
	  qDebug() << "Failed to execute " << sqlQuery;
	  qDebug() << query.lastError();
	  return 0;
	}
	qDebug() << query.lastInsertId();
	return query.lastInsertId().toInt();
  }
  return 0;
}

int Cellar::findOrInsertAppellation(const QString & name, const int country)
{
  if (name.isEmpty()) return 0;

  QSqlQuery query;
  QString sqlQuery = QString("SELECT id "
							 "FROM appellation "
							 "WHERE name=%1").arg(name);
  query.exec(sqlQuery);
  if (query.next()) {
	return query.value(0).toInt();
  } else {
	sqlQuery = QString("INSERT INTO appellation "
					   "(name, country_id) VALUES (\"%1\", %2);")
	  .arg(name).arg(country);

	if (!query.exec(sqlQuery)) {
	  qDebug() << "Failed to execute " << sqlQuery << ": " << query.lastError();
	  return 0;
	}
	qDebug() << query.lastInsertId();
	return query.lastInsertId().toInt();
  }
  return 0;
}

bool Cellar::importCSV(const QString & fileName,
					   const QString & separators,
					   const QChar delimiter,
					   const int startingLine,
					   const QHash<int, int> columnDefinition)
{
  QFile file(fileName);
  if (!file.open(QIODevice::ReadOnly)) {
	qDebug() << "Can not open file";
	return false;
  }

  qDebug() << "importCSV";
  CSVParser parser(separators, delimiter);
  int l = 1;
  QTextStream in(&file);
  QString line = in.readLine();
  QSqlQuery query;
  while (!line.isNull()) {
	if (l>=startingLine) {
	  QStringList list;
	  qDebug() << "line #" << l << ": " << line;
	  if (line.isEmpty()) goto NEXTLINE;
	  if (parser.parseLine(line, list)) {
		if (list.size() < WineColumn_Comment) goto NEXTLINE;
		QString name = text(list, columnDefinition, WineColumn_Name);
		int quantity = 
		  text(list, columnDefinition, WineColumn_Quantity).toInt();
		int boughtQuantity = 
		  text(list, columnDefinition, WineColumn_BoughtQuantity).toInt();
		float boughtPrice = 
		  text(list, columnDefinition, WineColumn_BoughtPrice).toDouble();
		QString boughtDate = "";
		int millesime = 
		  text(list, columnDefinition, WineColumn_Millesime).toInt();
		int bestBefore = 
		  text(list, columnDefinition, WineColumn_BestBefore).toInt();
		QString comment = 
		  text(list, columnDefinition, WineColumn_Comment);

		QString countryName =
		  text(list, columnDefinition, WineColumn_Country);
		int country = findOrInsert("country", countryName);

		QString placeName =
		  text(list, columnDefinition, WineColumn_Place);
		int place = findOrInsert("place", placeName);

		QString productorName =
		  text(list, columnDefinition, WineColumn_Productor);
		int productor = findOrInsert("productor", productorName);

		QString appellationName =
		  text(list, columnDefinition, WineColumn_Appellation);
		int appellation = findOrInsertAppellation(appellationName, country);

		int volume = 0;
		int category = 0;
		
		QString sqlQuery = "INSERT INTO wine ("
		  ""  "name,"
		  ""  "category_id,"
		  ""  "appellation_id,"
		  ""  "millesime,"
		  ""  "volume,"
		  ""  "quantity,"
		  ""  "country_id,"
		  ""  "productor_id,"
		  ""  "place_id,"
		  ""  "bought_date,"
		  ""  "bought_quantity,"
		  ""  "bought_price,"
		  ""  "best_before,"
		  ""  "comment"
		  ")"
		  " VALUES(";
		sqlQuery.append("\"" + name + "\",");
		sqlQuery.append(QString::number(category) + ",");
		sqlQuery.append(QString::number(appellation) + ",");
		sqlQuery.append(QString::number(millesime) + ",");
		sqlQuery.append(QString::number(volume) + ",");
		sqlQuery.append(QString::number(quantity) + ",");
		sqlQuery.append(QString::number(country) + ",");
		sqlQuery.append(QString::number(productor) + ",");
		sqlQuery.append(QString::number(place) + ",");
		if (boughtDate != "") {
		  QDate d = QDate::fromString(boughtDate, "d/m/yyyy");
		  sqlQuery.append("'" + d.toString(Qt::ISODate) + "',");
		} else {
		  sqlQuery.append("'" + boughtDate + "',");
		}
		sqlQuery.append(QString::number(boughtQuantity) + ",");
		sqlQuery.append(QString::number(boughtPrice) + ",");
		sqlQuery.append(QString::number(bestBefore) + ",");
		sqlQuery.append("\"" + comment + "\"");
		sqlQuery.append(")");
		
		qDebug() << sqlQuery;
		if (!query.exec(sqlQuery)) {
		  qDebug() << query.lastError();
		}
	  } else {
		qDebug() << "Failed to parse line #" << l;
	  }
	}

  NEXTLINE:
	// Read next line
	line = in.readLine();
	l++;
  }
  
  return true;
}

void Cellar::createSqlModel(void)
{
  m_model = new QSqlRelationalTableModel(this);
  m_model->setTable("wine");
  m_model->setHeaderData(WineColumn_Id, 
						 Qt::Horizontal, columnName(WineColumn_Id));
  m_model->setHeaderData(WineColumn_Category, 
						 Qt::Horizontal, columnName(WineColumn_Category));
  m_model->setHeaderData(WineColumn_Millesime,
						 Qt::Horizontal, columnName(WineColumn_Millesime));
  m_model->setHeaderData(WineColumn_Appellation,
						 Qt::Horizontal, columnName(WineColumn_Appellation));
  m_model->setHeaderData(WineColumn_Name,
						 Qt::Horizontal, columnName(WineColumn_Name));
  m_model->setHeaderData(WineColumn_Quantity,
						 Qt::Horizontal, columnName(WineColumn_Quantity));
  m_model->setHeaderData(WineColumn_BoughtQuantity, 
						 Qt::Horizontal, columnName(WineColumn_BoughtQuantity));
  m_model->setHeaderData(WineColumn_BoughtPrice,
						 Qt::Horizontal, columnName(WineColumn_BoughtPrice));
  m_model->setHeaderData(WineColumn_BoughtDate,
						 Qt::Horizontal, columnName(WineColumn_BoughtDate));
  m_model->setHeaderData(WineColumn_Country,
						 Qt::Horizontal, columnName(WineColumn_Country));
  m_model->setHeaderData(WineColumn_Productor,
						 Qt::Horizontal, columnName(WineColumn_Productor));
  m_model->setHeaderData(WineColumn_Volume,
						 Qt::Horizontal, columnName(WineColumn_Volume));
  m_model->setHeaderData(WineColumn_Place,
						 Qt::Horizontal, columnName(WineColumn_Place));
  m_model->setHeaderData(WineColumn_BestBefore,
						 Qt::Horizontal, columnName(WineColumn_BestBefore));
  m_model->setHeaderData(WineColumn_Comment,
						 Qt::Horizontal, columnName(WineColumn_Comment));
  
  m_model->setRelation(WineColumn_Appellation, 
			  QSqlRelation("appellation", "id", "name"));
  if (!m_model->relation(WineColumn_Appellation).isValid()) {
	qDebug() << "Relation not valid for appellation";
  }
  
  m_model->setRelation(WineColumn_Category, 
			  QSqlRelation("category", "id", "name"));
  if (!m_model->relation(WineColumn_Category).isValid()) {
	qDebug() << "Relation not valid for category";
  }
  
  m_model->setRelation(WineColumn_Volume, 
			  QSqlRelation("volume", "id", "name"));
  if (!m_model->relation(WineColumn_Volume).isValid()) {
	qDebug() << "Relation not valid for volume";
  }
  
  m_model->setRelation(WineColumn_Place, 
			  QSqlRelation("place", "id", "name"));
  if (!m_model->relation(WineColumn_Place).isValid()) {
	qDebug() << "Relation not valid for place";
  }
  
  m_model->setRelation(WineColumn_Country, 
			  QSqlRelation("country", "id", "name"));
  if (!m_model->relation(WineColumn_Country).isValid()) {
	qDebug() << "Relation not valid for country";
  }
  
  m_model->setRelation(WineColumn_Productor, 
			  QSqlRelation("productor", "id", "name"));
  if (!m_model->relation(WineColumn_Productor).isValid()) {
	qDebug() << "Relation not valid for productor";
  }

  m_model->select();

  // Create model for appellations
  m_appellationsModel = new QSqlRelationalTableModel(this);
  m_appellationsModel->setTable("appellation");
  m_appellationsModel->setRelation(AppellationColumn_Country, 
								   QSqlRelation("country", "id", "name"));
  m_appellationsModel->setHeaderData(AppellationColumn_Name, 
									 Qt::Horizontal, tr("Name"));
  m_appellationsModel->setHeaderData(AppellationColumn_Country, 
									 Qt::Horizontal, tr("Country"));
  m_appellationsModel->select();

  
  // Create model for productors
  m_productorsModel = new QSqlTableModel(this);
  m_productorsModel->setTable("productor");
  m_productorsModel->setHeaderData(ProductorColumn_Name, 
								   Qt::Horizontal, tr("Name"));
  m_productorsModel->setHeaderData(ProductorColumn_Address,
								   Qt::Horizontal, tr("Address"));
  m_productorsModel->setHeaderData(ProductorColumn_Email,
								   Qt::Horizontal, tr("Email"));
  m_productorsModel->setHeaderData(ProductorColumn_Phone,
								   Qt::Horizontal, tr("Phone"));
  m_productorsModel->setHeaderData(ProductorColumn_Fax,
								   Qt::Horizontal, tr("Fax"));

  // Create model for countries
  m_countriesModel = new QSqlTableModel(this);
  m_countriesModel->setTable("country");
  m_countriesModel->setHeaderData(CountryColumn_Name, 
								  Qt::Horizontal, tr("Name"));
  m_countriesModel->select();

  // Create model for places
  m_placesModel = new QSqlTableModel(this);
  m_placesModel->setTable("place");
  m_placesModel->setHeaderData(PlaceColumn_Name, Qt::Horizontal, tr("Name"));
  m_placesModel->select();

  // Create model for volumes
  m_volumesModel = new QSqlTableModel(this);
  m_volumesModel->setTable("volume");
  m_volumesModel->select();

  // Create model for categorys
  m_categoriesModel = new QSqlTableModel(this);
  m_categoriesModel->setTable("category");
  m_categoriesModel->select();
}

QString Cellar::tmpFileName(void)
{
  QTemporaryFile tmpFile;
  QString tmpFileName;
  if (tmpFile.open()) {
	tmpFileName = tmpFile.fileName();
	tmpFile.close();
	QFile::remove(tmpFileName);
	return tmpFileName;
  }
  return QString();
}

bool Cellar::convertFromXml(const QString & fileName)
{
  qDebug() << "convert file '" + fileName + "' from xml";
  QString tmpFileName = this->tmpFileName();
  if (QFile::rename(fileName, tmpFileName))
	{
	  if (!create(fileName))
		return false;
	  if (!convertAppellationsFromXml())
		goto error;
	  convertCountriesFromXml();
	  convertProductorsFromXml();
	  if (!convertWinesFromXml(tmpFileName))
		goto error;
	  return true;
	}
  qDebug() << "Can not copy to '" + tmpFileName + "'";
  return false;

 error:
  qDebug() << "conversion failed";
  if (!tmpFileName.isNull())
	QFile::rename(tmpFileName, fileName);
  return false;
}

bool Cellar::convertWinesFromXml(const QString & fileName)
{
  QDomDocument doc;
  int errorLine;
  int errorColumn;
  QString errorStr;
  QFile file(fileName);
  if (!doc.setContent(&file, true, &errorStr, &errorLine, &errorColumn)) {
	qDebug() << "Failed to open '" << file.fileName() << "'";
	return false;
  }
  
  QDomElement root = doc.documentElement();
  if (root.tagName() != "cellar") {
	qDebug() << "This is not the correct xml file";
	return false;
  }
  
  QSqlQuery query (m_db);
  QSet<int> placesList;
  // Create the new document
  QDomNode node = root.firstChildElement("wines").firstChild();
  while (!node.isNull()) {
	if (node.toElement().tagName() == "wine") {
	  QDomElement elm = node.toElement();
	  int id = elm.attributes().namedItem("ID").nodeValue().toInt();
	  QDomNode wineNode = elm.firstChild();
	  QString name;
	  int appellation = 0;
	  int quantity = 0;
	  int boughtQuantity = 0;
	  float boughtPrice = 0.0;
	  QString boughtDate = "";
	  int millesime = 0;
	  int category = 0;
	  int volume = 0;
	  int place = 0;
	  int country = -1;
	  int productor = 0;
	  int bestBefore = 0;
	  QString comment = "";
	  
	  while (!wineNode.isNull()) {
		if (wineNode.isElement()) {
		  QDomElement wineElm = wineNode.toElement();
		  QString value = wineElm.firstChild().nodeValue();
		  if (wineElm.tagName() == "name") {
			name = value;
		  } else if (wineElm.tagName() == "appellation") {
			appellation = value.toInt();
		  } else if (wineElm.tagName() == "quantity") {
			quantity = value.toInt();
		  } else if (wineElm.tagName() == "boughtQuantity") {
			boughtQuantity = value.toInt();
		  } else if (wineElm.tagName() == "boughtPrice") {
			boughtPrice = value.toFloat();
		  } else if (wineElm.tagName() == "boughtDate") {
			boughtDate = value;
		  } else if (wineElm.tagName() == "millesime") {
			millesime = value.toInt();
		  } else if (wineElm.tagName() == "category") {
			category = value.toInt();
		  } else if (wineElm.tagName() == "volume")	{
			volume = value.toInt();
		  } else if (wineElm.tagName() == "place") {
			place = value.toInt()+1;
			placesList.insert(place);
		  } else if (wineElm.tagName() == "productor") {
			productor = value.toInt();
		  } else if (wineElm.tagName() == "country") {
			country = value.toInt();
		  } else if (wineElm.tagName() == "bestBefore")	{
			bestBefore = value.toInt();
		  } else if (wineElm.tagName() == "comment") {
			comment = value;
		  } else {
			qDebug() << wineElm.tagName();
		  }
		}
		wineNode = wineNode.nextSibling();
	  }
	  
	  if ((country == -1) && (appellation != 0)) {
		// Resolve country from appellation
		QString sqlQuery = QString("SELECT country_id, name "
								   "FROM appellation "
								   "WHERE id=%1").arg(appellation);
		query.exec(sqlQuery);
		if (query.next()) {
		  country = query.value(0).toInt();
		  qDebug() << 
			QString("Resolved country from appellation #%1 '%2': %3")
			.arg(appellation).arg(query.value(1).toString()).arg(country);
		}
	  }
	  
	  if (bestBefore == 0)
		bestBefore = millesime;
	  
	  QString sqlQuery = "INSERT INTO wine ("
		""  "id,"
		""  "name,"
		""  "category_id,"
		""  "appellation_id,"
		""  "millesime,"
		""  "volume,"
		""  "quantity,"
		""  "country_id,"
		""  "productor_id,"
		""  "place_id,"
		""  "bought_date,"
		""  "bought_quantity,"
		""  "bought_price,"
		""  "best_before,"
		""  "comment"
		")"
		" VALUES(";
	  sqlQuery.append(QString::number(id) + ",");
	  sqlQuery.append("\"" + name + "\",");
	  sqlQuery.append(QString::number(category) + ",");
	  sqlQuery.append(QString::number(appellation) + ",");
	  sqlQuery.append(QString::number(millesime) + ",");
	  sqlQuery.append(QString::number(volume) + ",");
	  sqlQuery.append(QString::number(quantity) + ",");
	  sqlQuery.append(QString::number(country) + ",");
	  sqlQuery.append(QString::number(productor) + ",");
	  sqlQuery.append(QString::number(place) + ",");
	  if (boughtDate != "") {
		QDate d = QDate::fromString(boughtDate, "d/m/yyyy");
		sqlQuery.append("'" + d.toString(Qt::ISODate) + "',");
	  } else {
		sqlQuery.append("'" + boughtDate + "',");
	  }
	  sqlQuery.append(QString::number(boughtQuantity) + ",");
	  sqlQuery.append(QString::number(boughtPrice) + ",");
	  sqlQuery.append(QString::number(bestBefore) + ",");
	  sqlQuery.append("\"" + comment + "\"");
	  sqlQuery.append(")");
	  
	  if (!query.exec(sqlQuery)) {
		qDebug() << query.lastError();
		qDebug() << sqlQuery;
	  }
	}
	node = node.nextSibling();
  }
  
  foreach (int value, placesList) {
	QString sqlQuery = "INSERT INTO place (id, name) "
	  "VALUES ("
	  ""      + QString::number(value) + ","
	  ""      "\"Place #" + QString::number(value) + "\""
	  ")";
	
	if (!query.exec(sqlQuery)) {
	  qDebug() << query.lastError();
	  qDebug() << sqlQuery;
	}
  }
  
  return true;
}


bool Cellar::convertAppellationsFromXml(void)
{
  QDomDocument doc;
  int errorLine;
  int errorColumn;
  QString errorStr;
  QFile file("/home/jerome/.lecaviste/appellations.xml");
  if (!doc.setContent(&file, true, &errorStr, &errorLine, &errorColumn)) {
	qDebug() << "Failed to open '" << file.fileName() << "'";
	return false;
  }
  
  QDomElement root = doc.documentElement();
  if (root.tagName() != "appellations")	{
	qDebug() << "This is not the correct xml file";
	return false;
  }
  
  QDomNode node = root.firstChild();
  while (!node.isNull()) {
	if (node.toElement().tagName() == "appellation") {
	  QDomElement elm = node.toElement();
	  QDomNamedNodeMap atts = elm.attributes();
	  int id = atts.namedItem("ID").nodeValue().toInt();
	  int country = atts.namedItem("country").nodeValue().toInt();
	  QString name = elm.firstChild().toElement().firstChild().nodeValue();
	  QSqlQuery query (m_db);
	  QString sqlQuery = "INSERT INTO appellation ("
		""  "id,"
		""  "name,"
		""  "country_id"
		") VALUES("
		""  + QString::number(id) + ", "
		""  + "\"" + name + "\", "
		""  + QString::number(country) + ""
		")";
	  
	  if (!query.exec(sqlQuery)) {
		qDebug() << query.lastError();
		qDebug() << sqlQuery;
	  }
	}
	node = node.nextSibling();
  }
  
  return true;
}

bool Cellar::convertCountriesFromXml(void)
{
  QDomDocument doc;
  int errorLine;
  int errorColumn;
  QString errorStr;
  QFile file("/home/jerome/.lecaviste/countries.xml");
  if (!doc.setContent(&file, true, &errorStr, &errorLine, &errorColumn)) {
	qDebug() << "Failed to open '" << file.fileName() << "'";
	return false;
  }
  
  QDomElement root = doc.documentElement();
  if (root.tagName() != "countries") {
	qDebug() << "This is not the correct xml file";
	return false;
  }
  
  QDomNode node = root.firstChild();
  while (!node.isNull()) {
	if (node.toElement().tagName() == "country") {
	  QDomElement elm = node.toElement();
	  QDomNamedNodeMap atts = elm.attributes();
	  int id = atts.namedItem("ID").nodeValue().toInt();
	  QString name = elm.firstChild().toElement().firstChild().nodeValue();
	  QSqlQuery query (m_db);
	  QString sqlQuery = "INSERT INTO country ("
		""  "id,"
		""  "name"
		") VALUES("
		""  + QString::number(id) + ", "
		""  + "\"" + name + "\""
			")";
	  
	  if (!query.exec(sqlQuery)) {
		qDebug() << query.lastError();
		qDebug() << sqlQuery;
	  }
	}
	node = node.nextSibling();
  }
  
  return true;
}

bool Cellar::convertProductorsFromXml(void)
{
  QDomDocument doc;
  int errorLine;
  int errorColumn;
  QString errorStr;
  QFile file("/home/jerome/.lecaviste/productors.xml");
  if (!doc.setContent(&file, true, &errorStr, &errorLine, &errorColumn)) {
	qDebug() << "Failed to open '" << file.fileName() << "'";
	return false;
  }
  
  QDomElement root = doc.documentElement();
  if (root.tagName() != "productors") {
	qDebug() << "This is not the correct xml file";
	return false;
  }

  QDomNode node = root.firstChild();
  while (!node.isNull()) {
	if (node.toElement().tagName() == "productor") {
	  QDomElement elm = node.toElement();
	  QDomNamedNodeMap atts = elm.attributes();
	  int id = atts.namedItem("ID").nodeValue().toInt();
	  QString name;
	  QString address;
	  QString email;
	  QString phone;
	  QString fax;
	  QDomNode productorNode = elm.firstChild();
	  while (!productorNode.isNull()) {
		if (productorNode.isElement()) {
		  QDomElement wineElm = productorNode.toElement();
		  QString value = wineElm.firstChild().nodeValue();
		  if (wineElm.tagName() == "name") {
			name = value;
		  } else if (wineElm.tagName() == "address") {
			address = value;
		  } else if (wineElm.tagName() == "email") {
			email = value;
		  } else if (wineElm.tagName() == "phone") {
			phone = value;
		  } else if (wineElm.tagName() == "fax") {
			fax = value;
		  } else {
			qDebug() << wineElm.tagName();
		  }
		}
		productorNode = productorNode.nextSibling();
	  }
	  
	  QSqlQuery query (m_db);
	  QString sqlQuery = "INSERT INTO productor ("
		""  "id,"
		""  "name,"
		""  "address,"
		""  "email,"
		""  "phone,"
		""  "fax"
		") VALUES("
		""  + QString::number(id) + ", "
		""  + "\"" + name + "\","
		""  + "\"" + address + "\","
		""  + "\"" + email + "\","
		""  + "\"" + phone + "\","
		""  + "\"" + fax + "\""
		")";
	  
	  if (!query.exec(sqlQuery)) {
		qDebug() << query.lastError();
		qDebug() << sqlQuery;
	  }
	}
	node = node.nextSibling();
  }
  
  return true;
}

bool Cellar::check(void)
{
  QString findtableexist = "SELECT name, type, sql "
	"FROM sqlite_master "
	"WHERE type!='meta' ORDER BY name";
  QSqlQuery query(m_db);
  if (!query.exec(findtableexist))
	return false;

  return m_db.tables().contains("wine");
}

CellarStats Cellar::stats(void)
{
  // FIXME: This rule should be valid with any selection (i.e. should
  // be executed with m_model
  QString sqlQuery = "SELECT count(id), sum(quantity) FROM wine "
	"WHERE quantity>0";
  QSqlQuery query;
  if (!query.exec(sqlQuery)) {
	qDebug() << query.lastError();
	return CellarStats();
  } 
 
  if (query.next()) {
	return CellarStats(query.value(0).toInt(), query.value(1).toInt());
  }

  return CellarStats();
}

QSqlRecord Cellar::findAppellationByName(const QString & name)
{
  QSqlRecord rec;
  for (int r=0; r<appellations()->rowCount(); r++) {
	rec = appellations()->record(r);
	if (rec.value(AppellationColumn_Name).toString() == name) return rec;
  }

  return QSqlRecord();
}

QSqlRecord Cellar::findProductorByName(const QString & name)
{
  QSqlRecord rec;
  for (int r=0; r<productors()->rowCount(); r++) {
	rec = productors()->record(r);
	if (rec.value(ProductorColumn_Name).toString() == name) return rec;
  }

  return QSqlRecord();
}

QSqlRecord Cellar::findCountryByName(const QString & name)
{
  QSqlRecord rec;
  for (int r=0; r<countries()->rowCount(); r++) {
	rec = countries()->record(r);
	if (rec.value(CountryColumn_Name).toString() == name) return rec;
  }

  return QSqlRecord();
}

QSqlRecord Cellar::findCategoryByName(const QString & name)
{
  QSqlRecord rec;
  for (int r=0; r<categories()->rowCount(); r++) {
	rec = categories()->record(r);
	if (rec.value(CategoryColumn_Name).toString() == name) return rec;
  }

  return QSqlRecord();
}

QSqlRecord Cellar::findPlaceByName(const QString & name)
{
  QSqlRecord rec;
  for (int r=0; r<places()->rowCount(); r++) {
	rec = places()->record(r);
	QString s = rec.value(PlaceColumn_Name).toString();
	if (s == name) return rec;
  }
  return QSqlRecord();
}

QSqlRecord Cellar::findVolumeByName(const QString & name)
{
  QSqlRecord rec;
  for (int r=0; r<volumes()->rowCount(); r++) {
	rec = volumes()->record(r);
	if (rec.value(VolumeColumn_Name).toString() == name) return rec;
  }

  return QSqlRecord();
}

QSqlRecord Cellar::findAppellationById(const int id)
{
  QSqlRecord rec;
  for (int r=0; r<appellations()->rowCount(); r++) {
	rec = appellations()->record(r);
	if (rec.value(AppellationColumn_Id).toInt() == id) return rec;
  }

  return QSqlRecord();
}

QSqlRecord Cellar::findProductorById(const int id)
{
  QSqlRecord rec;
  for (int r=0; r<productors()->rowCount(); r++) {
	rec = productors()->record(r);
	if (rec.value(ProductorColumn_Id).toInt() == id) return rec;
  }

  return QSqlRecord();
}

QSqlRecord Cellar::findCountryById(const int id)
{
  QSqlRecord rec;
  for (int r=0; r<countries()->rowCount(); r++) {
	rec = countries()->record(r);
	if (rec.value(CountryColumn_Id).toInt() == id) return rec;
  }

  return QSqlRecord();
}

QSqlRecord Cellar::findCategoryById(const int id)
{
  QSqlRecord rec;
  for (int r=0; r<categories()->rowCount(); r++) {
	rec = categories()->record(r);
	if (rec.value(CategoryColumn_Id).toInt() == id) return rec;
  }

  return QSqlRecord();
}

QSqlRecord Cellar::findPlaceById(const int id)
{
  QSqlRecord rec;
  for (int r=0; r<places()->rowCount(); r++) {
	rec = places()->record(r);
	if (rec.value(PlaceColumn_Id).toInt() == id) return rec;
  }

  return QSqlRecord();
}

QSqlRecord Cellar::findVolumeById(const int id)
{
  QSqlRecord rec;
  for (int r=0; r<volumes()->rowCount(); r++) {
	rec = volumes()->record(r);
	if (rec.value(VolumeColumn_Id).toInt() == id) return rec;
  }

  return QSqlRecord();
}

QString Cellar::columnName(const WineColumn col)
{
  switch(col) {
  case WineColumn_Id: return tr("Id");
  case WineColumn_Name: return tr("Name");
  case WineColumn_Category: return tr("Category");
  case WineColumn_Appellation: return tr("Appellation");
  case WineColumn_Millesime: return tr("Millesime");
  case WineColumn_Volume: return tr("Volume");
  case WineColumn_Quantity: return tr("Quantity");
  case WineColumn_Country: return tr("Country");
  case WineColumn_Productor: return tr("Productor");
  case WineColumn_Place: return tr("Place");
  case WineColumn_BoughtDate: return tr("Bought date");
  case WineColumn_BoughtQuantity: return tr("Bought quantity");
  case WineColumn_BoughtPrice: return tr("Bought price");
  case WineColumn_BestBefore: return tr("Best before");
  case WineColumn_Comment: return tr("Comment");
  }

  return "";
}


QString Cellar::categoryName(const Category col)
{
  switch (col) {
  case Category_Red: return tr("red");
  case Category_White: return tr("white");
  case Category_Rose: return tr("rose");
  case Category_Champagne: return tr("champagne");
  case Category_Sweet: return tr("sweet");
  case Category_Cremant: return tr("cremant");
  case Category_Liqueur: return tr("liqueur");
  }
  return "";
}

QString Cellar::volumeName(const Volume vol)
{
  switch(vol) {
  case Volume_75cl: return tr("75cl");
  case Volume_Half: return tr("Half");
  case Volume_Magnum: return tr("Magnum");
  }
  return "";
}
