/* -*- c++ -*-
 *
 *  Le Caviste --- a wine cellar managing application
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  For more information, visit the Le Caviste Home Page at
 *  http://www.lecaviste.org/
 *
 *  Copyright (c) 2007
 *      Le Caviste Authors <devel@lecaviste.org>
 *
 */
#ifndef CELLAR_HPP
#define CELLAR_HPP

#include <QtSql>

class Cellar;

class CellarStats
{
  friend class Cellar;

public:
  bool isValid(void) const { return m_bottles != -1; }
  int bottles(void) const { return m_bottles; }
  int wines(void) const {return m_wines; }

protected:
  CellarStats(void);  
  CellarStats(int, int);  
  
private:
  int m_wines;
  int m_bottles;
};

inline QDebug operator<<(QDebug s, const CellarStats& stats) 
{
  s << "CellarStats(" << stats.bottles() << "," << stats.wines() << ")";
  return s;
}

class Cellar: public QObject
{
  Q_OBJECT

public:
  typedef enum {
	WineColumn_Id 				= 0,
	WineColumn_Name				= 1,
	WineColumn_Category  		= 2,
	WineColumn_Appellation  	= 3,
	WineColumn_Millesime		= 4,
	WineColumn_Volume			= 5,
	WineColumn_Quantity			= 6,
	WineColumn_Country  		= 7,
	WineColumn_Productor 		= 8,
	WineColumn_Place			= 9,
	WineColumn_BoughtDate		= 10,
	WineColumn_BoughtQuantity	= 11,
	WineColumn_BoughtPrice		= 12,
	WineColumn_BestBefore		= 13,
	WineColumn_Comment			= 14
  } WineColumn;

  enum {
	AppellationColumn_Id 		= 0,
	AppellationColumn_Name 		= 1,
	AppellationColumn_Country 	= 2
  };


  typedef enum {
	Category_Red 		= 0,
	Category_White		= 1,
	Category_Rose		= 2,
	Category_Champagne	= 3,
	Category_Sweet		= 4,
	Category_Cremant	= 5,
	Category_Liqueur	= 6
  } Category;

  typedef enum {
	Volume_75cl		= 0,
	Volume_Half 	= 1,
	Volume_Magnum	= 2
  } Volume;

  enum {
	PlaceColumn_Id 		= 0,
	PlaceColumn_Name 	= 1
  };

  enum {
	CountryColumn_Id 	= 0,
	CountryColumn_Name 	= 1
  };

  enum {
	CategoryColumn_Id 	= 0,
	CategoryColumn_Name 	= 1
  };

  enum {
	VolumeColumn_Id 	= 0,
	VolumeColumn_Name 	= 1
  };

  enum {
	ProductorColumn_Id 		= 0,
	ProductorColumn_Name 	= 1,
	ProductorColumn_Address = 2,
	ProductorColumn_Email 	= 3,
	ProductorColumn_Phone 	= 4,
	ProductorColumn_Fax 	= 5
  };

  Cellar(QObject * parent = 0);

  bool open(const QString & fileName);
  bool create(const QString & fileName);
  bool close(void);
  bool exportCSV(const QString & fileName);
  bool importCSV(const QString & fileName,
				 const QString & separators,
				 const QChar delimiter,
				 const int startingLine,
				 const QHash<int, int> columnDefinition);
  bool isOpened(void) const { return m_isOpened; }
  QString fileName(void) { return m_fileName; }

  QSqlRelationalTableModel * model(void) { return m_model; }
  QSqlTableModel * appellations(void) { return m_appellationsModel; }
  QSqlTableModel * places(void) { return m_placesModel; }
  QSqlTableModel * countries(void) { return m_countriesModel; }
  QSqlTableModel * productors(void) { return m_productorsModel; }
  QSqlTableModel * volumes(void) { return m_volumesModel; }
  QSqlTableModel * categories(void) { return m_categoriesModel; }

  CellarStats stats(void);

  void sort(WineColumn col) { 
	m_model->sort(col, Qt::AscendingOrder);
  }
  
  QSqlRecord findAppellationByName(const QString & name);
  QSqlRecord findCategoryByName(const QString & name);
  QSqlRecord findProductorByName(const QString & name);
  QSqlRecord findCountryByName(const QString & name);
  QSqlRecord findPlaceByName(const QString & name);
  QSqlRecord findVolumeByName(const QString & name);

  QSqlRecord findAppellationById(const int);
  QSqlRecord findCategoryById(const int);
  QSqlRecord findProductorById(const int);
  QSqlRecord findCountryById(const int);
  QSqlRecord findPlaceById(const int);
  QSqlRecord findVolumeById(const int);

  inline int findCategoryId(const QString & name) {
	QSqlRecord rec = findCategoryByName(name);
	if (rec.isEmpty()) return -1;
	return rec.value(CategoryColumn_Id).toInt();
  }
  
  inline int findAppellationId(const QString & name) {
	QSqlRecord rec = findAppellationByName(name);
	if (rec.isEmpty()) return -1;
	return rec.value(AppellationColumn_Id).toInt();
  }

  inline int findVolumeId(const QString & name) {
	QSqlRecord rec = findVolumeByName(name);
	if (rec.isEmpty()) return -1;
	return rec.value(VolumeColumn_Id).toInt();
  }

  inline int findProductorId(const QString & name) {
	QSqlRecord rec = findProductorByName(name);
	if (rec.isEmpty()) return -1;
	return rec.value(ProductorColumn_Id).toInt();
  }

  inline int findCountryId(const QString & name) {
	QSqlRecord rec = findCountryByName(name);
	if (rec.isEmpty()) return -1;
	return rec.value(CountryColumn_Id).toInt();
  }
  
  inline int findPlaceId(const QString & name) {
	QSqlRecord rec = findPlaceByName(name);
	if (rec.isEmpty()) return -1;
	return rec.value(PlaceColumn_Id).toInt();
  }
  
  static QString columnName(const WineColumn);
  static QString categoryName(const Category);
  static QString volumeName(const Volume);

protected:
  bool convertFromXml(const QString & fileName);
  bool convertWinesFromXml(const QString & fileName);
  bool convertAppellationsFromXml(void);
  bool convertCountriesFromXml(void);
  bool convertProductorsFromXml(void);
  bool check(void);
  void createSqlModel(void);
  QString tmpFileName(void);
  QString text(const QStringList & list,
			   const QHash<int, int> columnDefinition,
			   const WineColumn col);
  int findOrInsert(const QString & table, const QString & name);
  int findOrInsertAppellation(const QString & name, const int country);

private:
  QString 		m_fileName;
  QSqlDatabase	m_db;
  bool 			m_isOpened;
  QSqlRelationalTableModel *	m_model;
  QSqlRelationalTableModel *	m_appellationsModel;
  QSqlTableModel *	m_placesModel;
  QSqlTableModel *	m_countriesModel;
  QSqlTableModel *	m_productorsModel;
  QSqlTableModel *	m_volumesModel;
  QSqlTableModel *	m_categoriesModel;
};


#endif
