/* -*- c++ -*-
 *
 *  Le Caviste --- a wine cellar managing application
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  For more information, visit the Le Caviste Home Page at
 *  http://www.lecaviste.org/
 *
 *  Copyright (c) 2007
 *      Le Caviste Authors <devel@lecaviste.org>
 *
 */
#include "import_wizard.hpp"
#include "csv_parser.hpp"
#include "cellar.hpp"
#include "column_combobox.hpp"


ImportWizard::ImportWizard(QWidget * parent) : 
  QWizard(parent), m_startingLine(1)
{
  addPage(createSeparatorPage());
  addPage(createColumnAssociationPage());
  
  setWindowTitle(tr("Import Wizard"));

  for (int i=0; i<=Cellar::WineColumn_Comment; i++) {
	m_columnDefinition[i] = -1;
  }
}

QWizardPage * ImportWizard::createSeparatorPage(void)
{
  QWizardPage * page = new QWizardPage;
  page->setTitle(tr("Select separators"));
  
  QLabel * label = new QLabel(tr("Select separator which appear in the text."));
  label->setWordWrap(true);
  
  QGridLayout * layout = new QGridLayout;
  layout->addWidget(label, 0, 0, 1, 2);

  // Create group with check box for separator
  QGroupBox * separatorGroupBox = new QGroupBox;
  layout->addWidget(separatorGroupBox, 1, 0, 1, 2);
  QGridLayout * separatorLayout = new QGridLayout;
  separatorGroupBox->setLayout(separatorLayout);
  separatorGroupBox->setTitle(tr("Separator"));

  m_spaceBox = new QCheckBox(tr("Space"));
  separatorLayout->addWidget(m_spaceBox, 0, 0);
  m_tabularBox = new QCheckBox(tr("Tabular"));
  separatorLayout->addWidget(m_tabularBox, 0, 1);
  m_bangBox = new QCheckBox(tr("Bang"));
  separatorLayout->addWidget(m_bangBox, 0, 2);
  m_commaBox = new QCheckBox(tr("Comma (,)"));
  separatorLayout->addWidget(m_commaBox, 0, 3);
  m_colonBox = new QCheckBox(tr("Colon (:)"));
  separatorLayout->addWidget(m_colonBox, 1, 0);
  m_hyphenBox = new QCheckBox(tr("Hyphen (-)"));
  separatorLayout->addWidget(m_hyphenBox, 1, 1);
  m_semicolonBox = new QCheckBox(tr("Semicolon (;)"));
  separatorLayout->addWidget(m_semicolonBox, 1, 2);
  m_pipeBox = new QCheckBox(tr("Pipe (,)"));
  separatorLayout->addWidget(m_pipeBox, 1, 3);
  m_customBox = new QCheckBox(tr("Custom"));
  separatorLayout->addWidget(m_customBox, 2, 0);
  m_customField = new QLineEdit;
  m_customField->setEnabled(false);
  separatorLayout->addWidget(m_customField, 2, 1, 1, 2);
  
  connect(m_spaceBox, SIGNAL(clicked()),
		  this, SLOT(onSeparatorBoxClicked()));
  connect(m_tabularBox, SIGNAL(clicked()),
		  this, SLOT(onSeparatorBoxClicked()));
  connect(m_bangBox, SIGNAL(clicked()),
		  this, SLOT(onSeparatorBoxClicked()));
  connect(m_commaBox, SIGNAL(clicked()),
		  this, SLOT(onSeparatorBoxClicked()));
  connect(m_colonBox, SIGNAL(clicked()),
		  this, SLOT(onSeparatorBoxClicked()));
  connect(m_hyphenBox, SIGNAL(clicked()),
		  this, SLOT(onSeparatorBoxClicked()));
  connect(m_semicolonBox, SIGNAL(clicked()),
		  this, SLOT(onSeparatorBoxClicked()));
  connect(m_pipeBox, SIGNAL(clicked()),
		  this, SLOT(onSeparatorBoxClicked()));

  connect(m_customBox, SIGNAL(clicked()), 
		  this, SLOT(onCustomSeparatorClicked()));

  // Create group for text indicator
  QHBoxLayout * textIndicatorLayout = new QHBoxLayout;
  QGroupBox * textIndicatorGroupBox = new QGroupBox;
  textIndicatorGroupBox->setLayout(textIndicatorLayout);
  textIndicatorGroupBox->setTitle(tr("Text indicator"));
  m_doublequote = new QRadioButton(tr("Double-quote (\")"));
  m_doublequote->setChecked(true);
  textIndicatorLayout->addWidget(m_doublequote);
  m_quote = new QRadioButton(tr("Quote (')"));
  textIndicatorLayout->addWidget(m_quote);
  m_backquote = new QRadioButton(tr("Back-quote (`)"));
  textIndicatorLayout->addWidget(m_backquote);
  layout->addWidget(textIndicatorGroupBox, 2, 0, 1, 1);

  connect(m_doublequote, SIGNAL(clicked()),
		  this, SLOT(onTextDelimiterRadioClicked()));
  connect(m_quote, SIGNAL(clicked()),
		  this, SLOT(onTextDelimiterRadioClicked()));
  connect(m_backquote, SIGNAL(clicked()),
		  this, SLOT(onTextDelimiterRadioClicked()));


  // Create group for starting line
  QGroupBox * linesGroupBox = new QGroupBox;
  layout->addWidget(linesGroupBox, 2, 1, 1, 1);
  QHBoxLayout * linesLayout = new QHBoxLayout;
  linesGroupBox->setLayout(linesLayout);
  linesGroupBox->setTitle(tr("Lines to import"));
  QLabel * l1 = new QLabel(tr("Starting from"));
  linesLayout->addWidget(l1);
  m_startingLineSpinBox = new QSpinBox;
  m_startingLineSpinBox->setMinimum(1);
  linesLayout->addWidget(m_startingLineSpinBox);
  connect(m_startingLineSpinBox, SIGNAL(valueChanged(int)),
		  this, SLOT(onStartingLineChanged(int)));

  // Create group to show an example of separated columns
  QGroupBox * exampleGroupBox = new QGroupBox;
  exampleGroupBox->setTitle(tr("Example"));
  layout->addWidget(exampleGroupBox, 3, 0, 1, 2);
  m_exampleTable = new QTableWidget(0, 0);
  m_exampleTable->setEditTriggers(QAbstractItemView::NoEditTriggers);
  m_exampleTable->setSelectionMode(QAbstractItemView::NoSelection);
  m_exampleTable->horizontalHeader()->hide();
  m_exampleTable->verticalHeader()->hide();
  QVBoxLayout * layout2 = new QVBoxLayout;
  layout2->addWidget(m_exampleTable);
  exampleGroupBox->setLayout(layout2);

  page->setLayout(layout);

  return page;

}

QWizardPage * ImportWizard::createColumnAssociationPage(void)
{
  QWizardPage * page = new QWizardPage;
  page->setTitle("Association column/contents");
  
  QLabel * label = 
	new QLabel("Select correspondance between columns and thier content");
  label->setWordWrap(true);
  
  QVBoxLayout * layout = new QVBoxLayout;
  layout->addWidget(label);
  m_columnTable = new QTableWidget(0, 0);
  m_columnTable->horizontalHeader()->hide();
  m_columnTable->verticalHeader()->hide();
  m_columnTable->setEditTriggers(QAbstractItemView::NoEditTriggers);
  m_columnTable->setSelectionMode(QAbstractItemView::NoSelection);

  layout->addWidget(m_columnTable);

  page->setLayout(layout);
  
  return page;
}


void ImportWizard::initializePage(int id) 
{
  switch (id) {
  case SeparatorPage:
	updateModel();
	break;

  case ColumnAssociationPage:
	{
	  int rowCount = m_exampleTable->rowCount();
	  int colCount = m_exampleTable->columnCount();
	  m_columnTable->clear();
	  m_columnTable->setColumnCount(m_exampleTable->columnCount());
	  m_columnTable->setRowCount(rowCount+1);
	  
	  // Create header to selection column name
	  for(int c=0; c<colCount; c++) {
		ColumnComboBox * combo = new ColumnComboBox(c);
		m_columnTable->setCellWidget(0, c, combo);

		connect(combo, SIGNAL(currentIndexChanged(int)), 
				this, SLOT(onColumnAssociationChanged(int)));
	  }
	  
	  for (int r=0; r<rowCount; r++) {
		// Fill columns
		for(int c=0; c<colCount; c++) {
		  QTableWidgetItem * it = m_exampleTable->item(r, c);
		  if (!it) continue;
		  QString t = it->text();
		  QTableWidgetItem * it2 = new QTableWidgetItem(t);
		  if (r < m_startingLine-1) {
			it2->setForeground(QBrush(Qt::darkGray));
		  }
		  m_columnTable->setItem(r+1, c, it2);
		}
		m_columnTable->resizeColumnsToContents();
	  }
	  break;
	}
  }
}

void ImportWizard::onSeparatorBoxClicked(void)
{
  updateModel();
}

void ImportWizard::onTextDelimiterRadioClicked(void)
{
  updateModel();
}

void ImportWizard::onCustomSeparatorClicked(void)
{
  m_customField->setEnabled(m_customBox->isChecked());
  updateModel();
}

void ImportWizard::onStartingLineChanged(int val)
{
  m_startingLine = val;
  int rowCount = m_exampleTable->rowCount();
  int colCount = m_exampleTable->columnCount();

  for(int r=0; r<rowCount-1; r++) {
	for(int c=0; c<colCount; c++) {
	  QTableWidgetItem * it = m_exampleTable->item(r, c);
	  if (!it) continue;

	  if (r < m_startingLine-1) {
		it->setForeground(QBrush(Qt::darkGray));
	  } else {
		it->setForeground(QBrush(Qt::black));
	  }
	}
  }
}

void ImportWizard::onColumnAssociationChanged(int index) 
{
  ColumnComboBox * comboBox = qobject_cast<ColumnComboBox *>(sender());
  int idx = comboBox->columnIndex();
  int col = m_columnDefinition[index];
  if (col != -1) {
	if (QMessageBox::warning(
			this, tr("Import"), 
			tr("Field '%1' is already associated with column %2."
			   "Replace association?")
			.arg(Cellar::columnName(Cellar::WineColumn(index)))
			.arg(col),
			QMessageBox::Yes | QMessageBox::No) == QMessageBox::No) {
	  comboBox->setCurrentIndex(0);
	  return;
	}
	// Set undefined to combo related to column 
 	QWidget * w = m_columnTable->cellWidget(0, col);
	ColumnComboBox * it = qobject_cast<ColumnComboBox *>(w);
	it->setCurrentIndex(0);
  }
  m_columnDefinition[index] = idx;
}

void ImportWizard::updateModel(void)
{
  // Create the list of separator
  m_separators.clear();
  if (m_spaceBox->isChecked()) {
	m_separators.append(' ');
  }
  if (m_tabularBox->isChecked()) {
	m_separators.append('\t');
  }
  if (m_bangBox->isChecked()) {
	m_separators.append('!');
  }
  if (m_commaBox->isChecked()) {
	m_separators.append(',');
  }
  if (m_colonBox->isChecked()) {
	m_separators.append(':');
  }
  if (m_hyphenBox->isChecked()) {
	m_separators.append('-');
  }
  if (m_semicolonBox->isChecked()) {
	m_separators.append(';');
  }
  if (m_pipeBox->isChecked()) {
	m_separators.append('|');
  }
  if (m_customBox->isChecked()) {
	QString customSep = m_customField->text();
	if (!customSep.isEmpty()) {
	  m_separators.append(customSep.at(0));
	}
  }

  // Check for text delimiter
  if (m_doublequote->isChecked()) {
	m_delimiter = '\"';
  } else if (m_quote->isChecked()) {
	m_delimiter = '\'';
  } else if (m_backquote->isChecked()) {
	m_delimiter = '`';
  } else {
	qDebug() << "Should have a text delimiter";
  }

  QFile file(m_fileName);
  if (!file.open(QIODevice::ReadOnly)) {
	qDebug() << "Can not open file";
	return;
  }

  QTextStream in(&file);
  QString line = in.readLine();
  m_exampleTable->clear();
  m_exampleTable->setColumnCount(1);
  m_exampleTable->setRowCount(0);
  int r = 0;	// Table row index
  int startingLine = m_startingLineSpinBox->value()-1;

  // Create a vector to mark empty columns
  QVector<bool> filledColumn(2);
  filledColumn.fill(false);

  CSVParser parser(m_separators, m_delimiter);
  while (!line.isNull()) {
	QStringList list;
	if (!parser.parseLine(line, list)) {
	  if (isVisible()) {
		QMessageBox::warning(this, tr("Import"), 
							 tr("File \"%1\" is not a valid CSV file").arg(m_fileName));
		reject();
	  }
	}
	
	// Check column
	for(int c=0; c<list.count(); c++) {
	  QString t = list[c];
	}

	// Reset column count
	if (list.count() > m_exampleTable->columnCount()) {
	  m_exampleTable->setColumnCount(list.count());
	  int s = filledColumn.size();
	  filledColumn.resize(list.count());
	  for (int i=s; i<list.count(); i++) {
		filledColumn[i] = false;
	  }
	}
	
	// Add a new row
	m_exampleTable->insertRow(r);
	
	// Fill columns
	for(int c=0; c<list.count(); c++) {
	  QString t = list[c];
	  QTableWidgetItem * it = new QTableWidgetItem(t);
	  if (r < startingLine) {
		it->setForeground(QBrush(Qt::darkGray));
	  }
	  m_exampleTable->setItem(r, c, it);

	  if (!t.isEmpty()) {
		filledColumn[c] = true;
	  }
	}
	line = in.readLine();
	r++;
  }

  // Covert the list and remove empty columns
  for (int i=filledColumn.size()-1; i>=0; i--) {
	if (!filledColumn[i]) {
	  m_exampleTable->removeColumn(i);
	}
  }
  // Update the SpinBox with the number of lines read
  m_startingLineSpinBox->setMaximum(r);
  m_exampleTable->resizeColumnsToContents ();
}


