
#include <QDir>
#include "config.h"

#define _URL_ "http://www.lecaviste.org/"

#include "lecaviste_window.hpp"
#include "wine_dialog.hpp"
#include "appellation_dialog.hpp"
#include "appellations_dialog.hpp"
#include "productor_dialog.hpp"
#include "productors_dialog.hpp"
#include "country_dialog.hpp"
#include "countries_dialog.hpp"
#include "place_dialog.hpp"
#include "places_dialog.hpp"
#include "import_wizard.hpp"
#include "preferences_dialog.hpp"

LeCavisteWindow::LeCavisteWindow(void) : 
  m_currentDisplayMode(DisplayMode_Name)
{
  createUI();
  m_printer = new Printer(m_winesView, this);
  readSettings();
}

void LeCavisteWindow::closeEvent(QCloseEvent * event)
{
  if (m_cellar.isOpened())
	m_cellar.close();
  writeSettings();
  event->accept();
}

void LeCavisteWindow::updateFilter(void)
{
  m_cellar.model()->setFilter("");
  if (!m_preferences.displayNullQuantityWines()) {
	m_cellar.model()->setFilter("quantity > 0");
  }
}

void LeCavisteWindow::createSqlModel(void)
{
  m_winesView->setModel(m_cellar.model());
  updateFilter();
  m_winesView->setColumnHidden(Cellar::WineColumn_Id, true);
  m_winesView->setColumnHidden(Cellar::WineColumn_Productor, true);
  m_winesView->setColumnHidden(Cellar::WineColumn_Comment, true);
  m_winesView->setColumnHidden(Cellar::WineColumn_BoughtQuantity, true);
  m_winesView->setColumnHidden(Cellar::WineColumn_BoughtPrice, true);
  m_winesView->setColumnHidden(Cellar::WineColumn_BoughtDate, true);
  //m_winesView->setSortingEnabled(true);
  m_winesView->setEditTriggers(QAbstractItemView::NoEditTriggers);
  m_winesView->setSelectionBehavior(QAbstractItemView::SelectRows);
  m_winesView->setTextElideMode(Qt::ElideRight);
  m_winesView->setCurrentIndex(QModelIndex());

  connect(m_winesView, SIGNAL(doubleClicked(const QModelIndex &)),
		  this, SLOT(onWineDoubleClicked(const QModelIndex &)));

  connect(m_winesView->selectionModel(), 
		  SIGNAL(currentRowChanged(const QModelIndex &, const QModelIndex &)), 
		  this, SLOT(onCurrentRowChanged(const QModelIndex&)));

  QHeaderView * header = m_winesView->header();
  //  header->setClickable(true);
  //  header->setSortIndicatorShown(true);
  header->setAlternatingRowColors(true);
}

void LeCavisteWindow::changeDisplayMode(DisplayMode mode)
{
  QHeaderView * header = m_winesView->header();
  // Restore header order:
  // 	(id), name, cat, appellation, millesime, volume, qty,
  //	country, (prod), place, (bought date), (bought qty), (bought price),
  //	bestbefore, (comment)
  for (int i=0; i<header->count(); i++)
	{
	  if (i != header->visualIndex(i))
		header->moveSection(header->visualIndex(i), i);
	}

  switch (mode)
	{
	case DisplayMode_Name:
	  {
 		m_cellar.sort(Cellar::WineColumn_Name);
		updateSortActions(m_actionSortByName);
	  }
	  break;
	  
	case DisplayMode_Appellation:
	  {
		// Order should be appellation, name, cat, millesime, volume, qty,
		//					country, place, bestbefore
 		m_cellar.sort(Cellar::WineColumn_Appellation);
		header->moveSection(3, 0);
		updateSortActions(m_actionSortByAppellation);
	  }
	  break;
	  
	case DisplayMode_Category:
	  {
		// Order should be cat, name, appellation, millesime, volume, qty,
		//					country, place, bestbefore
  		m_cellar.sort(Cellar::WineColumn_Category);
		header->moveSection(2, 0);
		updateSortActions(m_actionSortByCategory);
	  }
	  break;
	  
	case DisplayMode_Place:
	  {
  		m_cellar.sort(Cellar::WineColumn_Place);
 		header->moveSection(9, 0);
 		header->moveSection(3, 7);
 		header->moveSection(5, 7);
		updateSortActions(m_actionSortByPlace);
	  }
	  break;
	  
	case DisplayMode_Country:
	  {
  		m_cellar.sort(Cellar::WineColumn_Country);
		header->moveSection(7, 0);
		updateSortActions(m_actionSortByCountry);
	  }
	  break;
	  
	default:
	  break;
	}

  m_currentDisplayMode = mode;
}

void LeCavisteWindow::onActionNew(void)
{
  QString fileName = 
	QFileDialog::getSaveFileName(this,
								 tr("Create cellar file"),
								 ".",
								 tr("Cellar file (*.cel)"));

  newFile(fileName);
}

void LeCavisteWindow::onActionOpen(void)
{
  QString fileName = 
	QFileDialog::getOpenFileName(this,
								 tr("Open cellar file"),
								 ".",
								 tr("Cellar file (*.cel)"));
  if (!fileName.isEmpty())
	loadFile(fileName);
}


void LeCavisteWindow::onActionOpenRecentFile(void)
{
  QAction * action = qobject_cast<QAction *>(sender());
  if (action)
	loadFile(action->data().toString());
}

void LeCavisteWindow::onActionImport(void)
{
  bool createNewCellar = true;
  if (m_cellar.isOpened()) {
	if (QMessageBox::question(this, tr("Import CSV file"),
							  tr("A cellar is already opened.\n"
								 "Do you want to create a new cellar?\n"
								 "Click Yes to create a new cellar\n"
								 "Click No to import wines in current cellar."),
							  QMessageBox::Yes | QMessageBox::Default,
							  QMessageBox::No | QMessageBox::Escape)
		== QMessageBox::No) {
	  createNewCellar = false;
	} else {
	  m_cellar.close();
	}
  } 
  
  QString fileName = 
	QFileDialog::getOpenFileName(this,
								   tr("Import a csv file"),
								 ".",
								 tr("CSV File (*.csv)"));

  // Manage cancel
  if (fileName.isEmpty()) return;

  if (createNewCellar) {
	// Create a new cellar
	QFileInfo fi(fileName);
	QString base = fi.completeBaseName();
	newFile(base);
  }

  ImportWizard * wizard = new ImportWizard(this);
  wizard->setTextFile(fileName);
  if (wizard->exec() == QDialog::Accepted) {
	qDebug() << "separators: " << wizard->separators();
	qDebug() << "startingLine: " << wizard->startingLine();
	qDebug() << "delimiter: " << wizard->delimiter();
	qDebug() << "columnDefinition: " << wizard->columnDefinition();
	
	m_cellar.importCSV(fileName,
					   wizard->separators(),
					   wizard->delimiter(),
					   wizard->startingLine(),
					   wizard->columnDefinition());
  }
  m_cellar.model()->select();
  updateStats();
}

void LeCavisteWindow::onActionExportCSV(void)
{
  QString fileName = 
	QFileDialog::getSaveFileName(this,
								 tr("Export cellar file"),
								 ".",
								 tr("CSV file (*.csv)"));

  if (fileName.isEmpty()) return;

  if (QFileInfo(fileName).suffix() != "csv") {
	fileName += ".csv";
  }
  
  if (!m_cellar.exportCSV(fileName)) {
	QMessageBox::warning(this, 
						 tr("Export cellar file"), 
						 tr("Can not export cellar to \"%1\" !").arg(fileName));
  }
}

void LeCavisteWindow::onActionPreferences(void)
{
  PreferencesDialog * dialog = new PreferencesDialog(m_preferences, this);
  dialog->exec();
  if (m_cellar.isOpened()) {
	updateFilter();
  }
}

void LeCavisteWindow::onActionPrint(void)
{
  m_printer->setSorted(m_currentDisplayMode != DisplayMode_Name);
  m_printer->print();
}

void LeCavisteWindow::onActionPrintPreview(void)
{
  m_printer->setSorted(m_currentDisplayMode != DisplayMode_Name);
  m_printer->preview();
}

void LeCavisteWindow::onActionPageSetup(void)
{
  m_printer->setSorted(m_currentDisplayMode != DisplayMode_Name);
  m_printer->setup();
}

void LeCavisteWindow::onActionAddWine(void)
{
  WineDialog * dialog = new WineDialog(&m_cellar, this);
  int accepted = dialog->exec();
  
  if (accepted == 1) {
	updateStats();
	//	int lastRow = m_cellar.rowCount() - 1;
// 	m_winesView->selectRow(lastRow);
// 	m_winesView->scrollToBottom();

  }
}

void LeCavisteWindow::onActionEditWine(void)
{
  QModelIndex index = m_winesView->currentIndex();
  if (!index.isValid()) return;

  WineDialog * dialog = new WineDialog(&m_cellar, index.row(), this);
  dialog->exec();
  m_winesView->setCurrentIndex(index);
  m_winesView->scrollTo(index);
  updateStats();
}


void LeCavisteWindow::onActionDeleteWine(void)
{
  QModelIndex index = m_winesView->currentIndex();
  if (!index.isValid()) return;

  // Get selected place infos
  QSqlRecord record = m_cellar.model()->record(index.row());
  QString name = record.value("name").toString();

  if (QMessageBox::question(this, tr("Delete wine"),
							tr("Delete wine \"%1\"?").arg(name),
							QMessageBox::Yes | QMessageBox::Default,
							QMessageBox::No | QMessageBox::Escape)
	  == QMessageBox::No) {
	return;
  }

  m_cellar.model()->removeRow(index.row());
  m_cellar.model()->submitAll();
  updateStats();
}

void LeCavisteWindow::onActionDrinkBottle(void)
{
  QSqlQuery query;
  QModelIndex index = m_winesView->currentIndex();
  if (!index.isValid()) return;
  int row = index.row();

  QSqlRecord record = m_cellar.model()->record(index.row());
  int id = record.value("id").toInt();
  
  QString sqlQuery = QString("UPDATE wine SET quantity=quantity-1 WHERE id=%1")
	.arg(id);
  query.exec(sqlQuery); 
  m_cellar.model()->select();
  QModelIndex newIndex = m_cellar.model()->index(row, 0);
  m_winesView->setCurrentIndex(newIndex);
  m_winesView->scrollTo(newIndex);
  updateStats();
}

void LeCavisteWindow::onWineDoubleClicked(const QModelIndex& index)
{
  int row = index.row();
  WineDialog * dialog = new WineDialog(&m_cellar, index.row(), this);
  dialog->exec();
  QModelIndex newIndex = m_cellar.model()->index(row, 0);
  m_winesView->setCurrentIndex(newIndex);
  m_winesView->scrollTo(newIndex);
  updateStats();
}


void LeCavisteWindow::onActionAddAppellation(void)
{
  AppellationDialog * dialog = new AppellationDialog(&m_cellar, this);
  dialog->exec();
}

void LeCavisteWindow::onActionAddProductor(void)
{
  ProductorDialog * dialog = new ProductorDialog(&m_cellar, this);
  dialog->exec();
}

void LeCavisteWindow::onActionAddCountry(void)
{
  CountryDialog * dialog = 
	new CountryDialog(&m_cellar, this);
  dialog->exec();
}

void LeCavisteWindow::onActionAddPlace(void)
{
  PlaceDialog * dialog = new PlaceDialog(&m_cellar, this);
  dialog->exec();
}

void LeCavisteWindow::onActionShowAppellations(void)
{
  AppellationsDialog * dialog = 
	new AppellationsDialog(&m_cellar, this);
  dialog->exec();
}

void LeCavisteWindow::onActionShowProductors(void)
{
  ProductorsDialog * dialog = new ProductorsDialog(&m_cellar, this);
  dialog->exec();
}

void LeCavisteWindow::onActionShowCountries(void)
{
  CountriesDialog * dialog = new CountriesDialog(&m_cellar, this);
  dialog->exec();
}

void LeCavisteWindow::onActionShowPlaces(void)
{
  PlacesDialog * dialog = new PlacesDialog(&m_cellar, this);
  dialog->exec();
}

void LeCavisteWindow::onActionCollapse(void)
{
  m_winesView->collapseAll();
}

void LeCavisteWindow::onActionExpand(void)
{
  m_winesView->expandAll();
}

void LeCavisteWindow::onActionSortByName(void)
{
  changeDisplayMode(DisplayMode_Name);
}

void LeCavisteWindow::onActionSortByAppellation(void)
{
  changeDisplayMode(DisplayMode_Appellation);
}

void LeCavisteWindow::onActionSortByCategory(void)
{
  changeDisplayMode(DisplayMode_Category);
}

void LeCavisteWindow::onActionSortByCountry(void)
{
  changeDisplayMode(DisplayMode_Country);
}

void LeCavisteWindow::onActionSortByPlace(void)
{
  changeDisplayMode(DisplayMode_Place);
}


void LeCavisteWindow::onActionAbout(void)
{
  QMessageBox::about(this, 
					 tr("About Le Caviste"),
					 QString("Le Caviste<br />"
					 "&copy;2007 Jrme Martin <br />"
					 "version %1<br />"
					 "<a href='%2'>%2</a>")
					 .arg(_VERSION_).arg(_URL_));
}

void LeCavisteWindow::onActionHelp(void)
{
  QMessageBox::warning(this, tr("Not implemented"), 
					   tr("This function is not yet implemented !"));
}

void LeCavisteWindow::onCurrentRowChanged(const QModelIndex& index)
{
  if (index.isValid()) {
	QSqlRecord record = m_cellar.model()->record(index.row());
	int qty = record.value("quantity").toInt();
  
	m_actionEditWine->setEnabled(true);
	m_actionDeleteWine->setEnabled(true);
	m_actionDrinkBottle->setEnabled(qty>0);
  } else {
	m_actionEditWine->setEnabled(false);
	m_actionDeleteWine->setEnabled(false);
	m_actionDrinkBottle->setEnabled(false);
  }
}


bool LeCavisteWindow::loadFile(const QString & fileName)
{
  bool firstCellar = true;
  if (m_cellar.isOpened()) {
	m_cellar.close();
	firstCellar = false;
  }

  if (!m_cellar.open(fileName)) {
	statusBar()->showMessage(tr("Error opening file \"%1\"!").arg(fileName),
							 5000);
	return false;
  }
  createSqlModel();

  statusBar()->showMessage(tr("Cellar file has been loaded !"), 2000);
  setCurrentFile(fileName);

  if (firstCellar) {
	// Update colomns size if this is a first cellar displayed
	readViewSettings();
  }

  updateStats();
  // Update display mode
  changeDisplayMode(m_currentDisplayMode);
  return true;
}

bool LeCavisteWindow::newFile(const QString & fileName)
{
  if (fileName.isEmpty()) return false;
  
  QString curFileName = fileName;

  if (QFileInfo(curFileName).suffix() != "cel") {
	curFileName += ".cel";
  }

  bool firstCellar = true;
  if (m_cellar.isOpened()) {
	m_cellar.close();	
	firstCellar = false;
  }

  QFile file(curFileName);
  if (file.exists()) {
	// Remove file first, we confirm in file selection dialog that we
	// want to remove this file
	file.remove();
  }

  m_cellar.create(curFileName);
  createSqlModel();
  setCurrentFile(curFileName);

  if (firstCellar) {
	// Update colomns size if this is a first cellar displayed
	readViewSettings();
  }

  return true;
}



void LeCavisteWindow::updateRecentFileActions(void)
{
  QMutableStringListIterator it(m_recentFiles);
  while (it.hasNext()) {
	if (!QFile::exists(it.next()))
	  it.remove();
  }

  for (int i=0; i<MaxRecentFiles; i++) {
	if (i < m_recentFiles.count()) {
	  QString showname = QFileInfo(m_recentFiles[i]).fileName();
	  QString text = tr("&%1 %2").arg(i+1).arg(showname);
	  m_actionOpenRecentFile[i]->setText(text);
	  m_actionOpenRecentFile[i]->setStatusTip(tr("Open file '%1'.")
											  .arg(m_recentFiles[i]));
	  m_actionOpenRecentFile[i]->setData(m_recentFiles[i]);
	  m_actionOpenRecentFile[i]->setVisible(true);
	} else {
	  m_actionOpenRecentFile[i]->setVisible(false);
	}
  }

  m_actionSepartor->setVisible(!m_recentFiles.isEmpty());
}


void LeCavisteWindow::setCurrentFile(const QString & filename)
{
  QString showname = tr("Untitled");

  if (!filename.isEmpty()) {
	showname = QFileInfo(filename).fileName();
	m_recentFiles.removeAll(filename);
	m_recentFiles.prepend(filename);
	updateRecentFileActions();
	updateMenus();
  }

  setWindowTitle(tr("%1 - Le Caviste").arg(showname));
}


void LeCavisteWindow::updateMenus(void) 
{
  m_actionExportCSV->setEnabled(true);
  m_actionPrint->setEnabled(true);
  m_actionPrintPreview->setEnabled(true);
  m_actionPageSetup->setEnabled(true);

  m_actionAddWine->setEnabled(true);
  m_actionAddAppellation->setEnabled(true);
  m_actionAddProductor->setEnabled(true);
  m_actionAddCountry->setEnabled(true);
  m_actionAddPlace->setEnabled(true);
  m_actionShowAppellations->setEnabled(true);
  m_actionShowProductors->setEnabled(true);
  m_actionShowCountries->setEnabled(true);
  m_actionShowPlaces->setEnabled(true);

//   m_actionExpand->setEnabled(true);
//   m_actionCollapse->setEnabled(true);
  m_actionSortByName->setEnabled(true);
  m_actionSortByAppellation->setEnabled(true);
  m_actionSortByCategory->setEnabled(true);
  m_actionSortByCountry->setEnabled(true);
  m_actionSortByPlace->setEnabled(true);
}

void LeCavisteWindow::updateSortActions(QAction * action)
{
  m_actionSortByName->setChecked(false);
  m_actionSortByAppellation->setChecked(false);
  m_actionSortByCategory->setChecked(false);
  m_actionSortByCountry->setChecked(false);
  m_actionSortByPlace->setChecked(false);

  action->setChecked(true);
}

void LeCavisteWindow::createUI(void)
{
  QWidget * 	centralwidget;
  QVBoxLayout *	vboxLayout;
  QMenu *		menuFile;
  QMenu *		menuWines;
  QMenu *		menuDisplay;
  QMenu *		menuHelp;
  int i;

  m_actionNew = new QAction(this);
  m_actionNew->setIcon(
	QIcon(QString::fromUtf8(":/images/icons/filenew.png")));
  
  m_actionOpen = new QAction(this);
  m_actionOpen->setIcon(
	QIcon(QString::fromUtf8(":/images/icons/fileopen.png")));
  
  for (i=0; i<MaxRecentFiles; i++) {
	m_actionOpenRecentFile[i] = new QAction(this);
	m_actionOpenRecentFile[i]->setVisible(true);
	connect(m_actionOpenRecentFile[i], SIGNAL(triggered()),
			this, SLOT(onActionOpenRecentFile()));
  }

  m_actionImport = new QAction(this);
  m_actionExportCSV = new QAction(this);
  m_actionExportCSV->setEnabled(false);

  m_actionPrint = new QAction(this);
  m_actionPrint->setIcon(
	QIcon(QString::fromUtf8(":/images/icons/printer.png")));
  m_actionPrint->setEnabled(false);
  m_actionPrintPreview = new QAction(this);
  m_actionPrintPreview->setIcon(
	QIcon(QString::fromUtf8(":/images/icons/print_preview.png")));
  m_actionPrintPreview->setEnabled(false);
  m_actionPageSetup = new QAction(this);
  m_actionPageSetup->setIcon(
	QIcon(QString::fromUtf8(":/images/icons/printer_setup.png")));
  m_actionPageSetup->setEnabled(false);

  m_actionPreferences = new QAction(this);

  m_actionQuit = new QAction(this);

  m_actionExpand = new QAction(this);
  m_actionExpand->setIcon(
	QIcon(QString::fromUtf8(":/images/icons/arrow_out.png")));
  m_actionExpand->setEnabled(false);

  m_actionCollapse = new QAction(this);
  m_actionCollapse->setIcon(
	QIcon(QString::fromUtf8(":/images/icons/arrow_in.png")));
  m_actionCollapse->setEnabled(false);

  m_actionAddWine = new QAction(this);
  m_actionAddWine->setEnabled(false);
  m_actionAddWine->setIcon(
	QIcon(QString::fromUtf8(":/images/icons/wine_add.png")));
  m_actionDeleteWine = new QAction(this);
  m_actionDeleteWine->setEnabled(false);
  m_actionDeleteWine->setIcon(
	QIcon(QString::fromUtf8(":/images/icons/wine_delete.png")));
  m_actionEditWine = new QAction(this);
  m_actionEditWine->setEnabled(false);
  m_actionEditWine->setIcon(
	QIcon(QString::fromUtf8(":/images/icons/wine_edit.png")));
  m_actionDrinkBottle = new QAction(this);
  m_actionDrinkBottle->setEnabled(false);
  m_actionDrinkBottle->setIcon(
	QIcon(QString::fromUtf8(":/images/icons/wine_go.png")));
  m_actionAddAppellation = new QAction(this);
  m_actionAddAppellation->setEnabled(false);
  m_actionAddProductor = new QAction(this);
  m_actionAddProductor->setEnabled(false);
  m_actionAddCountry = new QAction(this);
  m_actionAddCountry->setEnabled(false);
  m_actionAddPlace = new QAction(this);
  m_actionAddPlace->setEnabled(false);

  m_actionShowAppellations = new QAction(this);
  m_actionShowAppellations->setEnabled(false);
  m_actionShowProductors = new QAction(this);
  m_actionShowProductors->setEnabled(false);
  m_actionShowCountries = new QAction(this);
  m_actionShowCountries->setEnabled(false);
  m_actionShowPlaces = new QAction(this);
  m_actionShowPlaces->setEnabled(false);

  m_actionSortByName = new QAction(this);
  m_actionSortByName->setEnabled(false);
  m_actionSortByName->setCheckable(true);
  m_actionSortByAppellation = new QAction(this);
  m_actionSortByAppellation->setEnabled(false);
  m_actionSortByAppellation->setCheckable(true);
  m_actionSortByCategory = new QAction(this);
  m_actionSortByCategory->setEnabled(false);
  m_actionSortByCategory->setCheckable(true);
  m_actionSortByCountry = new QAction(this);
  m_actionSortByCountry->setEnabled(false);
  m_actionSortByCountry->setCheckable(true);
  m_actionSortByPlace = new QAction(this);
  m_actionSortByPlace->setEnabled(false);
  m_actionSortByPlace->setCheckable(true);

  m_actionAbout = new QAction(this);

  m_actionHelp = new QAction(this);
  m_actionHelp->setIcon(
	QIcon(QString::fromUtf8(":/images/icons/help.png")));

  centralwidget = new QWidget(this);
  vboxLayout = new QVBoxLayout(centralwidget);
  vboxLayout->setSpacing(2);
  vboxLayout->setMargin(2);
  m_winesView = new QTreeView(centralwidget);

  vboxLayout->addWidget(m_winesView);
  
  this->setCentralWidget(centralwidget);

  m_menubar = new QMenuBar(this);
  m_menubar->setGeometry(QRect(0, 0, 503, 29));
  menuFile = new QMenu(m_menubar);
  menuWines = new QMenu(m_menubar);
  menuDisplay = new QMenu(m_menubar);
  menuHelp = new QMenu(m_menubar);
  this->setMenuBar(m_menubar);

  m_statusbar = new QStatusBar(this);
  m_stats = new QLabel(tr("Empty"));
  m_statusbar->addPermanentWidget(m_stats);
  this->setStatusBar(m_statusbar);

  m_toolBar = new QToolBar(this);
  m_toolBar->setMovable(true);
  m_toolBar->setAllowedAreas(Qt::AllToolBarAreas);
  m_toolBar->setOrientation(Qt::Horizontal);
  this->addToolBar(static_cast<Qt::ToolBarArea>(4), m_toolBar);
  
  m_menubar->addAction(menuFile->menuAction());
  m_menubar->addAction(menuWines->menuAction());
  m_menubar->addAction(menuDisplay->menuAction());
  m_menubar->addAction(menuHelp->menuAction());

  menuHelp->addAction(m_actionAbout);
  menuHelp->addAction(m_actionHelp);

  menuFile->addAction(m_actionNew);
  menuFile->addAction(m_actionOpen);
  menuFile->addSeparator();
  for (i=0; i<MaxRecentFiles; i++)
	menuFile->addAction(m_actionOpenRecentFile[i]);
  m_actionSepartor = menuFile->addSeparator();
  menuFile->addAction(m_actionImport);
  menuFile->addAction(m_actionExportCSV);
  menuFile->addSeparator();
  menuFile->addAction(m_actionPreferences);
  menuFile->addSeparator();
  menuFile->addAction(m_actionPrint);
  menuFile->addAction(m_actionPrintPreview);
  menuFile->addAction(m_actionPageSetup);
  menuFile->addSeparator();
  menuFile->addAction(m_actionQuit);

  menuWines->addAction(m_actionAddWine);
  menuWines->addAction(m_actionDeleteWine);
  menuWines->addAction(m_actionEditWine);
  menuWines->addAction(m_actionDrinkBottle);
  menuWines->addSeparator();
  menuWines->addAction(m_actionAddAppellation);
  menuWines->addAction(m_actionAddProductor);
  menuWines->addAction(m_actionAddCountry);
  menuWines->addAction(m_actionAddPlace);
  menuWines->addSeparator();
  menuWines->addAction(m_actionShowAppellations);
  menuWines->addAction(m_actionShowProductors);
  menuWines->addAction(m_actionShowCountries);
  menuWines->addAction(m_actionShowPlaces);

  menuDisplay->addAction(m_actionExpand);
  menuDisplay->addAction(m_actionCollapse);
  menuDisplay->addSeparator();
  menuDisplay->addAction(m_actionSortByName);
  menuDisplay->addAction(m_actionSortByAppellation);
  menuDisplay->addAction(m_actionSortByCategory);
  menuDisplay->addAction(m_actionSortByCountry);
  menuDisplay->addAction(m_actionSortByPlace);

  m_toolBar->addAction(m_actionNew);
  m_toolBar->addAction(m_actionOpen);
  m_toolBar->addSeparator();
  m_toolBar->addAction(m_actionPrint);
  m_toolBar->addAction(m_actionPrintPreview);
  m_toolBar->addSeparator();
  m_toolBar->addAction(m_actionAddWine);
  m_toolBar->addAction(m_actionDeleteWine);
  m_toolBar->addAction(m_actionEditWine);
  m_toolBar->addAction(m_actionDrinkBottle);
  m_toolBar->addSeparator();
  m_toolBar->addAction(m_actionExpand);
  m_toolBar->addAction(m_actionCollapse);
  
  QSize size(503, 468);
  size = size.expandedTo(this->minimumSizeHint());
  this->resize(size);
  
  m_appName = "Le Caviste";
  this->setWindowTitle(m_appName);
  m_actionOpen->setText(tr("Open..."));
  m_actionOpen->setStatusTip(tr("Open a cellar file."));
  m_actionOpen->setShortcut(QKeySequence(tr("Ctrl+O")));

  m_actionNew->setText(tr("New..."));
  m_actionNew->setStatusTip(tr("Create a new cellar file."));
  m_actionNew->setShortcut(QKeySequence(tr("Ctrl+N")));

  m_actionImport->setText(tr("Import..."));
  m_actionImport->setStatusTip(tr("Import a cellar from CSV file."));

  m_actionExportCSV->setText(tr("Export..."));
  m_actionExportCSV->setStatusTip(tr("Export cellar to a CSV file."));

  m_actionPreferences->setText(tr("Preferences ..."));
  m_actionPreferences->setStatusTip(tr("Edit preferences."));

  m_actionPrint->setText(tr("Print ..."));
  m_actionPrint->setStatusTip(tr("Print cellar."));

  m_actionPrintPreview->setText(tr("Preview ..."));
  m_actionPrintPreview->setStatusTip(tr("Print preview."));

  m_actionPageSetup->setText(tr("Page setup ..."));
  m_actionPageSetup->setStatusTip(tr("Page setup."));

  m_actionQuit->setText(tr("Quit"));
  m_actionQuit->setStatusTip(tr("Quit application."));
  m_actionQuit->setShortcut(QKeySequence(tr("Ctrl+Q")));

  m_actionAddWine->setText(tr("Add a wine..."));
  m_actionAddWine->setStatusTip(tr("Add a new wine to the list."));

  m_actionDeleteWine->setText(tr("Delete a wine"));
  m_actionDeleteWine->setStatusTip(tr("Delete the selected wine from the list."));

  m_actionEditWine->setText(tr("Edit a wine..."));
  m_actionEditWine->setStatusTip(tr("Edit the selected wine."));

  m_actionDrinkBottle->setText(tr("Drink a bottle"));
  m_actionDrinkBottle->setStatusTip(tr("Drink a bottle of the selected wine."));

  m_actionAddAppellation->setText(tr("Add an appellation..."));
  m_actionAddAppellation->setStatusTip(tr("Add a new appellation to the list."));

  m_actionAddProductor->setText(tr("Add a productor..."));
  m_actionAddProductor->setStatusTip(tr("Add a new productor to the list."));

  m_actionAddCountry->setText(tr("Add a country..."));
  m_actionAddCountry->setStatusTip(tr("Add a new country to the list."));

  m_actionAddPlace->setText(tr("Add a place..."));
  m_actionAddPlace->setStatusTip(tr("Add a new place to the list."));

  m_actionShowAppellations->setText(tr("Show appellations..."));
  m_actionShowAppellations->setStatusTip(tr("Show list of appellations."));

  m_actionShowProductors->setText(tr("Show productors..."));
  m_actionShowProductors->setStatusTip(tr("Show list of productors."));

  m_actionShowCountries->setText(tr("Show countries..."));
  m_actionShowCountries->setStatusTip(tr("Show list of countries."));

  m_actionShowPlaces->setText(tr("Show places..."));
  m_actionShowPlaces->setStatusTip(tr("Show list of places."));

  m_actionExpand->setText(tr("Expand"));
  m_actionExpand->setStatusTip(tr("Expand wines tree."));

  m_actionCollapse->setText(tr("Collapse"));
  m_actionCollapse->setStatusTip(tr("Collapse wines tree."));

  m_actionSortByName->setText(tr("Sort by name"));
  m_actionSortByName->setStatusTip(tr("Sort wines by name."));

  m_actionSortByAppellation->setText(tr("Sort by appellation"));
  m_actionSortByAppellation->setStatusTip(tr("Sort wines by appellation."));

  m_actionSortByCategory->setText(tr("Sort by category"));
  m_actionSortByCategory->setStatusTip(tr("Sort wines by category."));

  m_actionSortByCountry->setText(tr("Sort by country"));
  m_actionSortByCountry->setStatusTip(tr("Sort wines by country."));

  m_actionSortByPlace->setText(tr("Sort by place"));
  m_actionSortByPlace->setStatusTip(tr("Sort wines by place."));

  m_actionAbout->setText(tr("About"));
  m_actionAbout->setStatusTip(tr("Display about box."));

  m_actionHelp->setText(tr("Help"));
  m_actionHelp->setToolTip(tr("Help"));
  m_actionHelp->setStatusTip(tr("Display online help."));
  m_actionHelp->setShortcut(QKeySequence(tr("Ctrl+?")));


  menuHelp->setTitle(tr("Help"));
  menuFile->setTitle(tr("File"));
  menuWines->setTitle(tr("Wines"));
  menuDisplay->setTitle(tr("Display")); 

  
  // Create connection
  connect(m_actionQuit, SIGNAL(triggered()), this, SLOT(close()));
  connect(m_actionOpen, SIGNAL(triggered()), 
		  this, SLOT(onActionOpen()));
  connect(m_actionNew, SIGNAL(triggered()), 
		  this, SLOT(onActionNew()));
  connect(m_actionImport, SIGNAL(triggered()), 
		  this, SLOT(onActionImport()));
  connect(m_actionExportCSV, SIGNAL(triggered()), 
		  this, SLOT(onActionExportCSV()));
  connect(m_actionPreferences, SIGNAL(triggered()), 
		  this, SLOT(onActionPreferences()));
  connect(m_actionPrint, SIGNAL(triggered()), 
		  this, SLOT(onActionPrint()));
  connect(m_actionPrintPreview, SIGNAL(triggered()), 
		  this, SLOT(onActionPrintPreview()));
  connect(m_actionPageSetup, SIGNAL(triggered()), 
		  this, SLOT(onActionPageSetup()));

  connect(m_actionAddWine, SIGNAL(triggered()),
		  this, SLOT(onActionAddWine()));
  connect(m_actionDeleteWine, SIGNAL(triggered()),
		  this, SLOT(onActionDeleteWine()));
  connect(m_actionEditWine, SIGNAL(triggered()),
		  this, SLOT(onActionEditWine()));
  connect(m_actionDrinkBottle, SIGNAL(triggered()),
		  this, SLOT(onActionDrinkBottle()));
  connect(m_actionAddAppellation, SIGNAL(triggered()),
		  this, SLOT(onActionAddAppellation()));
  connect(m_actionAddProductor, SIGNAL(triggered()),
		  this, SLOT(onActionAddProductor()));
  connect(m_actionAddCountry, SIGNAL(triggered()),
		  this, SLOT(onActionAddCountry()));
  connect(m_actionAddPlace, SIGNAL(triggered()),
		  this, SLOT(onActionAddPlace()));

  connect(m_actionShowAppellations, SIGNAL(triggered()),
		  this, SLOT(onActionShowAppellations()));
  connect(m_actionShowProductors, SIGNAL(triggered()),
		  this, SLOT(onActionShowProductors()));
  connect(m_actionShowCountries, SIGNAL(triggered()),
		  this, SLOT(onActionShowCountries()));
  connect(m_actionShowPlaces, SIGNAL(triggered()),
		  this, SLOT(onActionShowPlaces()));

  connect(m_actionCollapse, SIGNAL(triggered()), 
		  this, SLOT(onActionCollapse()));
  connect(m_actionExpand, SIGNAL(triggered()), 
		  this, SLOT(onActionExpand()));
  connect(m_actionSortByName, SIGNAL(triggered()),
		  this, SLOT(onActionSortByName()));
  connect(m_actionSortByAppellation, SIGNAL(triggered()),
		  this, SLOT(onActionSortByAppellation()));
  connect(m_actionSortByCategory, SIGNAL(triggered()),
		  this, SLOT(onActionSortByCategory()));
  connect(m_actionSortByCountry, SIGNAL(triggered()),
		  this, SLOT(onActionSortByCountry()));
  connect(m_actionSortByPlace, SIGNAL(triggered()),
		  this, SLOT(onActionSortByPlace()));

  connect(m_actionAbout, SIGNAL(triggered()), 
		  this, SLOT(onActionAbout()));
  connect(m_actionHelp, SIGNAL(triggered()), 
		  this, SLOT(onActionHelp()));
}

void LeCavisteWindow::updateStats(void)
{
  CellarStats s = m_cellar.stats();
  if (s.isValid()) {
	m_stats->setText(tr("%1 wines/%2 bottles").arg(s.wines()).arg(s.bottles()));
  } else {
	m_stats->setText(tr("Empty"));
  }
}


void LeCavisteWindow::writeSettings(void)
{
  QSettings settings("lecaviste.org", "Le Caviste");
  settings.setValue("geometry", geometry());
  settings.setValue("recentFiles", m_recentFiles);
  settings.setValue("latestFile", m_cellar.fileName());
  settings.setValue("openLatestCellar", m_preferences.openLatestCellar());
  settings.setValue("displayNullQuantityWines", 
					m_preferences.displayNullQuantityWines());

  settings.setValue("displayMode", m_currentDisplayMode);
  
  if (m_cellar.fileName() != "") {
	// Do not save column with if no file has been opened then col
	// with is 0
	settings.beginGroup("ColumnsGeometry");
	for (int i=0; i<ColumnsCount; i++) {
	  if (!m_winesView->isColumnHidden(i)) {
		settings.setValue(QString("col%1").arg(i,2, 10, QChar('0')), 
						  m_winesView->columnWidth(i));
	  }
	}
	settings.endGroup();
  }
}


void LeCavisteWindow::readSettings(void)
{
  bool b;
  QSettings settings("lecaviste.org", "Le Caviste");

  m_recentFiles = settings.value("recentFiles").toStringList();
  updateRecentFileActions();

  QRect rect = settings.value("geometry", QRect(200, 200, 400, 400)).toRect();
  move(rect.topLeft());
  resize(rect.size());

  
  b = settings.value("displayNullQuantityWines", false).toBool();
  m_preferences.setDisplayNullQuantityWines(b);

  // This should be the latest thing because loadFile may used other params
  b = settings.value("openLatestCellar", false).toBool();
  m_preferences.setOpenLatestCellar(b);
  if (b && !m_cellar.isOpened()) {
	QString fileName = settings.value("latestFile").toString();
	if (!fileName.isEmpty()) {
	  loadFile(fileName);
	  readViewSettings();
	}
  }
}


void LeCavisteWindow::readViewSettings(void)
{
  QSettings settings("lecaviste.org", "Le Caviste");

  QString mode = settings.value("DisplayMode", "").toString();
  if (mode != "") {
   	changeDisplayMode((DisplayMode)mode.toInt());
  } else {
    changeDisplayMode(DisplayMode_Name);
  }
  
  settings.beginGroup("ColumnsGeometry");
  for (int i=0; i<ColumnsCount; i++) {
	int w = settings.value(QString("col%1").arg(i,2, 10, QChar('0')), 100)
	  .toInt();
	m_winesView->setColumnWidth(i, w);
  }
  settings.endGroup();
}

