/* -*- c++ -*-
 *
 *  Le Caviste --- a wine cellar managing application
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  For more information, visit the Le Caviste Home Page at
 *  http://www.lecaviste.org/
 *
 *  Copyright (c) 2007
 *      Le Caviste Authors <devel@lecaviste.org>
 *
 */
#include "preview_view.hpp"

PreviewView::PreviewView(QPrinter & printer)
  :  m_printer(printer)
{
  verticalScrollBar()->setSingleStep(20);
  horizontalScrollBar()->setSingleStep(20);
  
  viewport()->setBackgroundRole(QPalette::Dark);
  
  m_scale = 1.0;
  m_interPageSpacing = 30;
  
  setup();
}

void PreviewView::zoomIn()
{
  m_scale += 0.2;
  resizeEvent(0);
  viewport()->update();
}

void PreviewView::zoomOut()
{
  m_scale -= 0.2;
  resizeEvent(0);
  viewport()->update();
}

void PreviewView::paintEvent(QPaintEvent *)
{
  QPainter p(viewport());
  p.translate(-horizontalScrollBar()->value(), -verticalScrollBar()->value());
  p.translate(m_interPageSpacing, m_interPageSpacing);
  
  for (int i = 0; i < m_pageCount; ++i) {
	p.save();
	p.scale(m_scale, m_scale);
	
	paintPage(p, i);
	
	p.restore();
	p.translate(0, m_interPageSpacing + m_paperSize.height()*m_scale);
  }
}

void PreviewView::paintPage(QPainter& painter, int p)
{
  QColor col(Qt::black);
  
  painter.setPen(col);
  painter.setBrush(Qt::white);
  painter.drawRect(QRectF(QPointF(0, 0), m_paperSize));
  painter.setBrush(Qt::NoBrush);
  
  col = col.light();
  painter.drawLine(QLineF(m_paperSize.width(), 1,
						  m_paperSize.width(), m_paperSize.height() - 1));
  
  col = col.light();
  painter.drawLine(QLineF(m_paperSize.width(), 2,
						  m_paperSize.width(), m_paperSize.height() - 2));
  
  painter.translate(m_pageTopLeft);

  emit pageRequested(p, painter);
}

void PreviewView::resizeEvent(QResizeEvent *)
{
  const QSize viewportSize = viewport()->size();
  
  QSize docSize;
  docSize.setWidth(qRound(m_paperSize.width() * m_scale + 2 * m_interPageSpacing));
  docSize.setHeight(qRound(m_pageCount * m_paperSize.height() * m_scale + (m_pageCount + 1) * m_interPageSpacing));
  
  horizontalScrollBar()->setRange(0, docSize.width() - viewportSize.width());
  horizontalScrollBar()->setPageStep(viewportSize.width());
  
  verticalScrollBar()->setRange(0, docSize.height() - viewportSize.height());
  verticalScrollBar()->setPageStep(viewportSize.height());
}

void PreviewView::mousePressEvent(QMouseEvent *e)
{
  m_mousePressPos = e->pos();
  m_scrollBarValuesOnMousePress.rx() = horizontalScrollBar()->value();
  m_scrollBarValuesOnMousePress.ry() = verticalScrollBar()->value();
  e->accept();
}

void PreviewView::mouseMoveEvent(QMouseEvent *e)
{
  if (m_mousePressPos.isNull()) {
	e->ignore();
	return;
  }
  
  horizontalScrollBar()->setValue(m_scrollBarValuesOnMousePress.x() 
								  - e->pos().x() + m_mousePressPos.x());
  verticalScrollBar()->setValue(m_scrollBarValuesOnMousePress.y() 
								- e->pos().y() + m_mousePressPos.y());
  horizontalScrollBar()->update();
  verticalScrollBar()->update();
  e->accept();
}

void PreviewView::mouseReleaseEvent(QMouseEvent *e)
{
  m_mousePressPos = QPoint();
  e->accept();
}


void PreviewView::setNumberOfPages(int count)
{
  m_pageCount = count;
}

void PreviewView::setup(void)
{
  m_paperSize = m_printer.paperRect().size();
  m_paperSize.rwidth() *= qreal(logicalDpiX()) / m_printer.logicalDpiX();
  m_paperSize.rheight() *= qreal(logicalDpiY()) / m_printer.logicalDpiY();
  
  m_pageTopLeft = m_printer.pageRect().topLeft();
  m_pageTopLeft.rx() *= qreal(logicalDpiX()) / m_printer.logicalDpiX();
  m_pageTopLeft.ry() *= qreal(logicalDpiY()) / m_printer.logicalDpiY();
}
