/* -*- c++ -*-
 *
 *  Le Caviste --- a wine cellar managing application
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  For more information, visit the Le Caviste Home Page at
 *  http://www.lecaviste.org/
 *
 *  Copyright (c) 2007
 *      Le Caviste Authors <devel@lecaviste.org>
 *
 */
#include <math.h>

#include "printer.hpp"
#include "preview_window.hpp"
#include "preview_view.hpp"
#include "print_page.hpp"

#include <QtGlobal>
#define DEBUG_PRINTER 1

Printer::Printer(QTreeView * winesView, QWidget * parent)
  : m_parent(parent), m_winesView(winesView), m_sorted(false)
{
  m_printer = new QPrinter(); 
  m_headerView = winesView->header();
  
  // Init default page setup
  m_pageOrder = TopToBottom;
  m_fontSize = 12;
  m_scale = 0; // this mean fit to width

#ifdef Q_OS_WIN32
  // Under windows, this is set by page setup
  m_marginLeft = 0;
  m_marginRight = 0;
  m_marginTop = 0;
  m_marginBottom = 0;
#else
  m_marginLeft = mmToPixels(10, m_printer);
  m_marginRight = mmToPixels(10, m_printer);
  m_marginTop = mmToPixels(10, m_printer);
  m_marginBottom = mmToPixels(10, m_printer);
#endif
}

Printer::~Printer(void)
{
  delete m_printer;
}

void Printer::print(void)
{
  // Get a first estimation to compute page number
  paginate(*m_printer);
  QPrintDialog printDialog(m_printer, m_parent);
  printDialog.setFromTo(1, m_pages.size());
  printDialog.setMinMax(1, m_pages.size());
  if (printDialog.exec()) {
	QPainter painter;
	if (!painter.begin(m_printer)) {
	  QMessageBox::warning(m_parent, tr("Printing error"),
						   tr("There is a printing problem. No printer detected."),
						   QMessageBox::Ok);
	  return;
	}
	// FIXME: paginate should be called by changed in printDialog
	paginate(*m_printer);

	int firstPage = m_printer->fromPage()-1;
	if (firstPage >= m_pages.size()) return;
	if (firstPage == -1) firstPage = 0;

	int lastPage = m_printer->toPage() -1;
	if ((lastPage == -1) || (lastPage >= m_pages.size()))
	  lastPage = m_pages.size();

	int numCopies = m_printer->numCopies();
#if DEBUG_PRINTER
	qDebug() << "Printing from" << firstPage+1 << "to" << lastPage;
	qDebug() << "  " << numCopies << "copies";
#endif

	int numPages = lastPage - firstPage;
	for (int i=0; i<numCopies; ++i) {
	  for (int j=0; j<numPages; ++j) {
		if ((i != 0) || (j != 0)) {
		  m_printer->newPage();
		}

		int index;
		if (m_printer->pageOrder() == QPrinter::FirstPageFirst) {
		  index = firstPage + j;
		} else {
		  index = lastPage - j;
		}
		
#if DEBUG_PRINTER
		qDebug() << "  Printing #" << index+1;
#endif
		printPage(index, painter);
	  }
	}
	painter.end();
  }
}

void Printer::preview(void)
{
  paginate(*m_printer);
  
  PreviewWindow * preview = new PreviewWindow(*m_printer, m_parent);
  preview->view()->setNumberOfPages(m_pages.size());
  connect(preview->view(),
		  SIGNAL(pageRequested(int, QPainter &)),
		  this, 
		  SLOT(printPage(int, QPainter &)),
		  Qt::DirectConnection);
 
  connect(this,
		  SIGNAL(pagesCountChanged(int)),
		  preview->view(),
		  SLOT(setNumberOfPages(int)),
		  Qt::DirectConnection);
 
  connect(preview,
		  SIGNAL(printRequired()),
		  this,
		  SLOT(print()));

  connect(preview,
		  SIGNAL(setupRequired()),
		  this,
		  SLOT(setup()));
  preview->setWindowModality(Qt::WindowModal);
  preview->setAttribute(Qt::WA_DeleteOnClose);
  preview->show();
}

void Printer::setup(void)
{
  QPageSetupDialog dlg(m_printer, m_parent);
  dlg.exec();
  paginate(*m_printer);
}

int Printer::computeColsWidth(QPainter * painter) 
{
  QFont font = painter->font();
  int rows = m_model->rowCount();
  int sum = 0;
  // Compute cols width by interating over all lines
  m_colsWidth.clear();

  int cols = m_model->columnCount ();

  // Consider header
  font.setBold(true);
  painter->setFont(font);  
  for (int c=0; c<cols; c++) {
	int v = m_headerView->logicalIndex(c);
	QString f = m_model->headerData(v, Qt::Horizontal).toString();
	if (!m_winesView->isColumnHidden(v)) {
	  QRect r = painter->boundingRect(0, 0, m_pageWidth, m_pageHeight, 
									  Qt::AlignLeft, f);
	  int w = r.width()+2*smallGap;
	  m_colsWidth.append(w);
	} else {
	  m_colsWidth.append(0);
	}
  }
  // Restore unbolded font
  font.setBold(false);
  painter->setFont(font);  
  
  for (int i=0; i<rows; i++) {
	QSqlRecord record = m_model->record(i);
	for (int c=0; c<cols; c++) {
	  int v = m_headerView->logicalIndex(c);
	  if (!m_winesView->isColumnHidden(v)) {
		QString f = record.value(v).toString();
		QRect r = painter->boundingRect(0, 0, m_pageWidth, m_pageHeight, 
										  Qt::AlignLeft, f);
		int w = r.width()+2*smallGap;
		m_colsWidth[c] = qMax(m_colsWidth[c], w);
	  }
	}
  }

  // Compute the total width for an eventually adaptation
  for (int c=0; c<m_colsWidth.size(); c++) {
	sum += m_colsWidth[c];
  }

  return sum;
}

void Printer::paginate(QPrinter &printer)
{
#if DEBUG_PRINTER
  qDebug() << "Paginate";
#endif

  m_model = (QSqlRelationalTableModel*)m_winesView->model();

  m_pageTotalWidth = printer.pageRect().width();
  m_pageWidth = m_pageTotalWidth - m_marginLeft - m_marginRight;
  m_pageTotalHeight = printer.pageRect().height();
  m_pageHeight = m_pageTotalHeight - m_marginTop - m_marginBottom;
  m_currentMarginLeft = m_marginLeft;
  
  QPixmap pixmap(printer.pageRect().size());
  QPainter painter(&pixmap);
  QFont font = painter.font();

  int rows = m_model->rowCount();

  if (m_scale == 0) {
	// Compute with current size
	font.setPointSize(m_fontSize);
	painter.setFont(font);
	int w = computeColsWidth(&painter);
	if (w > m_pageWidth) {
	  // iterate with size decrement until a correct size is found
#if DEBUG_PRINTER
	  qDebug() << "  Look for smaller font size to have" 
			   << w << "<" << m_pageWidth;
#endif

	  while ((w > m_pageWidth) && (m_fontSize > 1)){
		m_fontSize--;
		font.setPointSize(m_fontSize);
		painter.setFont(font);
		w = computeColsWidth(&painter);
#if DEBUG_PRINTER
		qDebug() << "    " << m_fontSize << w;
#endif
	  }
	} else {
	  // iterate with size decrement until a correct size is found
#if DEBUG_PRINTER
	  qDebug() << "  Look for larger font size to have" 
			   << w << ">" << m_pageWidth;
#endif

	  while ((w < m_pageWidth) && (m_fontSize <= 14)) {
		m_fontSize++;
		font.setPointSize(m_fontSize);
		painter.setFont(font);
		w = computeColsWidth(&painter);
	  }
	  // Now, w > m_pageWidth with current fontSize but previous was
	  // correct, consequently decrement font size and update font
	  if (m_pageWidth != w) m_fontSize--;
	  font.setPointSize(m_fontSize);
	  painter.setFont(font);
	  w = computeColsWidth(&painter);
	}
#if DEBUG_PRINTER
	qDebug() << "    Font size" << m_fontSize << "for width" << w;
#endif
	// Update marginLeft to center tabular in page
	m_currentMarginLeft = m_marginLeft + qRound((m_pageWidth - w)/2.0);
#if DEBUG_PRINTER
	qDebug() << "    margin Left" << m_currentMarginLeft;
#endif
	
  } else {
	m_fontSize *= qRound(m_scale/100.0);
#if DEBUG_PRINTER
	qDebug() << "    Font size" << m_fontSize;
#endif
	font.setPointSize(m_fontSize);
	painter.setFont(font);
	computeColsWidth(&painter);
  }

  // Get line height and compute number of lines per page and number
  // of pages needed
  QRect r = painter.boundingRect(0, 0, m_pageWidth, m_pageHeight, 
									Qt::AlignLeft, "x");
  m_lineHeight = r.height();
  // Remove one row for col header
  m_linesPerPage = m_pageHeight/(m_lineHeight+2*smallGap) - 1;
  int pagesForRows = ceil(rows*1./m_linesPerPage);

  // Compute columns
  QList<int> columnsPages;
  int x=0;
  int c;
  for (c=0; c<m_colsWidth.size(); c++) {
	if ((x + m_colsWidth[c]) > m_pageWidth) {
	  columnsPages.append(c-1);
	  x = 0;
	}
	x += m_colsWidth[c];
  }
  
  columnsPages.append(c-1);

  m_pages.clear();
  if (m_sorted) {
 	// Compute the position of rows in pages
	int cols = m_model->columnCount ();
	for (int c=0; c<cols; c++) {
	  m_logicalSortedCol = m_headerView->logicalIndex(c);
	  if (m_colsWidth[c] != 0) break;
	}
#if DEBUG_PRINTER
	qDebug() << "Sorted col:" << m_logicalSortedCol;
#endif
 	QString groupName;
	PrintPage currentPage;
	int totalBottles = 0;
	int totalWines = 0;
  	for (int i=0; i<rows; i++) {
 	  QSqlRecord record = m_model->record(i);
 	  QString f = record.value(m_logicalSortedCol).toString();

 	  if (!groupName.isEmpty() && (f != groupName)) {
		// This is a new group
		if (totalBottles > 1) {
		  currentPage.append(QString(tr("Total: %1 bottles / %2 wines"))
							 .arg(totalBottles).arg(totalWines));
		} else {
		  currentPage.append(QString());
		}
		if (currentPage.rowsCount() == m_linesPerPage) {
		  // Create a new page
		  for (int c=0; c<columnsPages.size(); c++) {
			int colStart = 0;
			if (c!=0) colStart = columnsPages[c-1]+1;
			int colEnd = columnsPages[c];
			currentPage.setColRange(colStart, colEnd);
			
			m_pages.append(currentPage);
		  }
		  currentPage.clear();
		}
		totalBottles = 0;
		totalWines = 0;
 	  }
	  currentPage.append(i);
	  if (currentPage.rowsCount() == m_linesPerPage) {
		// Create a new page
		for (int c=0; c<columnsPages.size(); c++) {
		  int colStart = 0;
		  if (c!=0) colStart = columnsPages[c-1]+1;
		  int colEnd = columnsPages[c];
		  currentPage.setColRange(colStart, colEnd);
		  
		  m_pages.append(currentPage);
		}
		currentPage.clear();
	  }
 	  groupName = f;
	  totalBottles++;
	  totalWines += record.value(Cellar::WineColumn_Quantity).toInt();
 	}

	// Add the last page
	currentPage.append(QString(tr("Total: %1 bottles / %2 wines"))
					   .arg(totalBottles).arg(totalWines));
	// Create a new page
	for (int c=0; c<columnsPages.size(); c++) {
	  int colStart = 0;
	  if (c!=0) colStart = columnsPages[c-1]+1;
	  int colEnd = columnsPages[c];
	  currentPage.setColRange(colStart, colEnd);
	  
	  m_pages.append(currentPage);
	}
  } else {
	for (int p=0; p<pagesForRows; p++) {
	  PrintPage currentPage;
	  int rowStart=p*m_linesPerPage;
	  int rowEnd = qMin(rowStart+m_linesPerPage-1, rows-1);
	  currentPage.setRowRange(rowStart, rowEnd);
	  for (int c=0; c<columnsPages.size(); c++) {
		int colStart = 0;
		if (c!=0) colStart = columnsPages[c-1]+1;
		int colEnd = columnsPages[c];
		currentPage.setColRange(colStart, colEnd);
		m_pages.append(currentPage);
	  }
	}
  }


#if DEBUG_PRINTER
  PrintPage page;
  qDebug() << "  page list";
  foreach(page, m_pages) {
	qDebug() << "    " << page;
  }
  qDebug() << "-----";
#endif
  
  emit pagesCountChanged(m_pages.size());
}


void Printer::printPage(int pageNumber, QPainter &painter)
{
  painter.save();
  QFont font = painter.font();
  font.setPointSize(m_fontSize);
  painter.setFont(font);
  painter.translate(m_currentMarginLeft, m_marginTop+smallGap);
  PrintPage p = m_pages[pageNumber];

  painter.setPen(QPen(Qt::black, 2, Qt::SolidLine));
  int tabularWidth = 0;
  int firstColWidth = 0;
  font.setBold(true);
  painter.setFont(font);  
  for (int c=p.colStart(); c<=p.colEnd(); c++) {
	int v = m_headerView->logicalIndex(c);
	if (!m_winesView->isColumnHidden(v)) {
	  QString f = m_model->headerData(v, Qt::Horizontal).toString();
	  int colWidth = m_colsWidth[c];
	  if (firstColWidth == 0) firstColWidth = colWidth;
	  QRect textRect = painter.boundingRect(smallGap, 0,
											 colWidth, m_lineHeight,
											 Qt::AlignLeft, f);
	  painter.drawText(textRect, Qt::AlignLeft, f);
	  painter.translate(colWidth, 0);
	  tabularWidth += colWidth;	  
	}
  }
  // restore to bold
  font.setBold(false);
  painter.setFont(font);

  painter.translate(-tabularWidth, m_lineHeight+smallGap);
  painter.drawLine(0, 0, tabularWidth, 0);
  painter.translate(0, smallGap);

  int nbRows = 0;
  QList<int> endGroup;
  QVariant r;
  QString groupName;
  foreach (r, p.rows()) {
	int startingPos = 1;
	if (r.type() == QVariant::String) {
	  QString s = r.toString();
	  if (!s.isEmpty() && !s.isNull()) {
		// Change font
		font.setItalic(true);
		painter.setFont(font);

		QRect textRect = painter.boundingRect(firstColWidth+smallGap, 0,
											   tabularWidth, m_lineHeight,
											   Qt::AlignLeft, s);
		painter.drawText(textRect, Qt::AlignLeft, s);
		painter.translate(tabularWidth, 0);
		// Restore font
		font.setItalic(false);
		painter.setFont(font);

		// The group ends after this row
		endGroup.append(nbRows+1);
	  } else {
		// This is the ending row
		//endGroup.append(nbRows);
		// Redraw the line completly
		painter.translate(0, -smallGap);
		painter.drawLine(0, 0, tabularWidth, 0);
		painter.translate(0, smallGap);
		continue;
	  }
	} else {
	  int i = r.toInt();
	  QSqlRecord record = m_model->record(i);
	  int x = 0;
	  for (int c=p.colStart(); c<=p.colEnd(); c++) {
		int v = m_headerView->logicalIndex(c);
		if (!m_winesView->isColumnHidden(v)) {
		  QString f = record.value(v).toString();
		  int colWidth = m_colsWidth[c];
		  x += colWidth;

		  if (v == m_logicalSortedCol) {
			if (!groupName.isEmpty() && (groupName == f)) {
			  f = "";
			} else {
			  groupName = f;
			}
			startingPos = firstColWidth+1;
		  }
		  QRect textRect = painter.boundingRect(smallGap, 0,
												 colWidth, m_lineHeight,
												 Qt::AlignLeft, f);
		  painter.drawText(textRect, Qt::AlignLeft, f);
		  painter.translate(colWidth, 0);
		}
	  }
	}

	painter.translate(-tabularWidth, m_lineHeight+smallGap);
	painter.drawLine(startingPos, 0, tabularWidth, 0);
	painter.translate(0, smallGap);

	nbRows++;
  }

  // Draw the last line revert previous smallGap translation and draw the line
  painter.translate(0, -smallGap);
  painter.drawLine(0, 0, tabularWidth, 0);

  // Add the last ending row
  endGroup.append(nbRows+1);

  // Start new drawing
  painter.restore();
  painter.save();

  // Draw first line
  painter.translate(m_currentMarginLeft, m_marginTop);
  painter.drawLine(0, 0, tabularWidth, 0);

  // Compute the real height for all rows + col header
  int tabularHeight = (nbRows+1)*(m_lineHeight+2*smallGap);

  // Draw the col
  painter.drawLine(0, 0, 0, tabularHeight);

  // Draw col line foreach group
  for (int g=0; g<endGroup.size(); g++) {
	int s = 0;
	if (g!=0) s = endGroup[g-1]+1;
	int e = endGroup[g];
	int start = s*(m_lineHeight+2*smallGap);
	int end = e*(m_lineHeight+2*smallGap);

	int x = 0;
	for (int c=p.colStart(); c<=p.colEnd(); c++) {
	  int v = m_headerView->logicalIndex(c);
	  if (!m_winesView->isColumnHidden(v)) {
		x += m_colsWidth[c];
		if (v == m_logicalSortedCol) {
		  // The col line after the title is complete
		  painter.drawLine(x, 0, x, tabularHeight);
		} else {
		  painter.drawLine(x, start, x, end);
		}
	  }
	}
  }
  // Draw last column 
  painter.drawLine(tabularWidth, 0, tabularWidth, tabularHeight);

  painter.restore();

  // Draw footer
  painter.drawText(QRect(0, 0, m_pageTotalWidth, m_pageTotalHeight), 
				   Qt::AlignHCenter | Qt::AlignBottom,
				   QString::number(pageNumber+1));
}
