/* -*- c++ -*-
 *
 *  Le Caviste --- a wine cellar managing application
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  For more information, visit the Le Caviste Home Page at
 *  http://www.lecaviste.org/
 *
 *  Copyright (c) 2007
 *      Le Caviste Authors <devel@lecaviste.org>
 *
 */
#include "wine_dialog.hpp"
#include "cellar.hpp"
#include "productor_dialog.hpp"
#include "appellation_dialog.hpp"
#include "place_dialog.hpp"
#include "country_dialog.hpp"


WineDialog::WineDialog(Cellar * cellar, QWidget * parent)
  : QDialog(parent), m_cellar(cellar), m_wineIdx(-1)
{
  m_model = cellar->model();

  createUI();
  m_title = tr("Add Wine");
  setWindowTitle(tr("Add wine"));
}

WineDialog::WineDialog(
	Cellar * cellar, 
	int index, 
	QWidget * parent)
  : QDialog(parent), m_cellar(cellar), m_wineIdx(index)
{
  m_model = cellar->model();

  createUI();
  fillWine();
#ifdef DEBUG
  QSqlRecord record = m_model->record(m_wineIdx);
  int id = record.value("id").toInt();
  m_title = tr("Edit wine (%1)").arg(id);
#else
  m_title = tr("Edit wine");
#endif
  setWindowTitle(tr("Edit wine"));
}

void WineDialog::createUI(void)
{ 
  QGroupBox * box = new QGroupBox(m_title);

  QLabel * nameLabel = new QLabel(tr("Name:"));
  QLabel * millesimeLabel = new QLabel(tr("Millesime:"));
  QLabel * categoryLabel = new QLabel(tr("Category"));
  QLabel * appellationLabel = new QLabel(tr("Appellation:"));
  QLabel * quantityLabel = new QLabel(tr("Quantity:"));
  QLabel * boughtQuantityLabel = new QLabel(tr("Bought Quantity:"));
  QLabel * boughtPriceLabel = new QLabel(tr("Bought Price:"));
  QLabel * boughtDateLabel = new QLabel(tr("Bought Date:"));
  QLabel * volumeLabel = new QLabel(tr("Volume:"));
  QLabel * productorLabel = new QLabel(tr("Productor:"));
  QLabel * countryLabel = new QLabel(tr("Country:"));
  QLabel * placeLabel = new QLabel(tr("Place:"));
  QLabel * bestBeforeLabel = new QLabel(tr("Best Before:"));
  QLabel * commentLabel = new QLabel(tr("Comment:"));
  
  QPushButton * addAppellation = new QPushButton("+");
  connect(addAppellation, SIGNAL(clicked()), 
		  this, SLOT(onAddAppellation()));
  QPushButton * addProductor = new QPushButton("+");
  connect(addProductor, SIGNAL(clicked()), 
		  this, SLOT(onAddProductor()));
  QPushButton * addCountry = new QPushButton("+");
  connect(addCountry, SIGNAL(clicked()), 
		  this, SLOT(onAddCountry()));
  QPushButton * addPlace = new QPushButton("+");
  connect(addPlace, SIGNAL(clicked()), 
		  this, SLOT(onAddPlace()));

  m_name = new QLineEdit;
  m_millesime = new QSpinBox;
  m_millesime->setMinimum(1990);
  m_millesime->setMaximum(QDate::currentDate().year());
  m_millesime->setValue(m_millesime->maximum());
  m_millesime->setReadOnly(false);
  
  m_millesimeCheckBox = new QCheckBox;
  connect(m_millesimeCheckBox, SIGNAL(stateChanged(int)), 
		  this, SLOT(onMillesimeCheckBoxChanged(int)));

  m_category = new QComboBox;
  m_cellar->categories()->setFilter("");
  m_category->setModel(m_cellar->categories());
  m_category->setModelColumn(1);
  m_category->setCurrentIndex(0);

  m_appellation = new QComboBox;
  m_cellar->appellations()->setFilter("");
  m_appellation->setModel(m_cellar->appellations());
  m_appellation->setModelColumn(1);
  m_appellation->model()->sort(1);
  m_appellation->setCurrentIndex(0);

  m_quantity = new QSpinBox;
  m_quantity->setValue(6);

  m_boughtQuantity = new QSpinBox;
  m_boughtQuantity->setValue(6);

  m_boughtPrice = new QLineEdit;

  m_boughtDate = new QDateEdit;
  m_boughtDate->setDate(QDate::currentDate());
  m_boughtDate->setDisplayFormat(tr("d MMMM yyyy"));

  m_volume = new QComboBox;
  m_cellar->volumes()->setFilter("");
  m_volume->setModel(m_cellar->volumes());
  m_volume->setModelColumn(1);
  m_volume->setCurrentIndex(0);

  m_productor = new QComboBox;
  m_cellar->productors()->setFilter("");
  m_productor->setModel(m_cellar->productors());
  m_productor->setModelColumn(1);
  m_productor->model()->sort(1);
  m_productor->setCurrentIndex(0);

  m_country = new QComboBox;
  m_cellar->countries()->setFilter("");
  m_country->setModel(m_cellar->countries());
  m_country->setModelColumn(1);
  m_country->model()->sort(1);
  m_country->setCurrentIndex(0);

  m_place = new QComboBox;
  m_cellar->places()->setFilter("");
  m_place->setModel(m_cellar->places());
  m_place->setModelColumn(1);
  m_place->setCurrentIndex(0);

  m_bestBefore = new QSpinBox;
  m_bestBefore->setMinimum(1990);
  m_bestBefore->setMaximum(QDate::currentDate().year()+30);
  m_bestBefore->setValue(QDate::currentDate().year());
  m_bestBefore->setReadOnly(false);

  m_comment = new QTextEdit;

  QGridLayout * boxLayout = new QGridLayout;
  int idx = 0;
  boxLayout->addWidget(nameLabel, idx, 0);
  boxLayout->addWidget(m_name, idx++, 1);

  boxLayout->addWidget(millesimeLabel, idx, 0);
  boxLayout->addWidget(m_millesime, idx, 1);
  boxLayout->addWidget(m_millesimeCheckBox, idx++, 2);

  boxLayout->addWidget(categoryLabel, idx, 0);
  boxLayout->addWidget(m_category, idx++, 1);

  boxLayout->addWidget(appellationLabel, idx, 0);
  boxLayout->addWidget(m_appellation, idx, 1);
  boxLayout->addWidget(addAppellation, idx, 2);
  idx++;

  if (m_wineIdx != -1) {
	boxLayout->addWidget(quantityLabel, idx, 0);
	boxLayout->addWidget(m_quantity, idx++, 1);
  }

  boxLayout->addWidget(volumeLabel, idx, 0);
  boxLayout->addWidget(m_volume, idx++, 1);

  boxLayout->addWidget(productorLabel, idx, 0);
  boxLayout->addWidget(m_productor, idx, 1);
  boxLayout->addWidget(addProductor, idx, 2);
  idx++;

  boxLayout->addWidget(countryLabel, idx, 0);
  boxLayout->addWidget(m_country, idx, 1);
  boxLayout->addWidget(addCountry, idx, 2);
  idx++;

  boxLayout->addWidget(placeLabel, idx, 0);
  boxLayout->addWidget(m_place, idx, 1);
  boxLayout->addWidget(addPlace, idx, 2);
  idx++;

  boxLayout->addWidget(boughtDateLabel, idx, 0);
  boxLayout->addWidget(m_boughtDate, idx++, 1);

  boxLayout->addWidget(boughtQuantityLabel, idx, 0);
  boxLayout->addWidget(m_boughtQuantity, idx++, 1);

  boxLayout->addWidget(boughtPriceLabel, idx, 0);
  boxLayout->addWidget(m_boughtPrice, idx++, 1);

  boxLayout->addWidget(bestBeforeLabel, idx, 0);
  boxLayout->addWidget(m_bestBefore, idx++, 1);

  boxLayout->addWidget(commentLabel, idx, 0);
  boxLayout->addWidget(m_comment, idx++, 1);
  box->setLayout(boxLayout);
   
  connect(m_appellation, SIGNAL(currentIndexChanged(const QString&)),
		  this, SLOT(onAppellationChanged(const QString&)));

  QPushButton * closeButton;
  QPushButton * validateButton;
 // Create buttons
  if (m_wineIdx == -1) {
  	// This is adition
  	closeButton = new QPushButton(tr("&Close (no save)"));
  	validateButton = new QPushButton(tr("&Save and close"));
 	} else {
	// This is edition
  	closeButton = new QPushButton(tr("&Cancel"));
  	validateButton = new QPushButton(tr("&Ok"));
  }
  
  closeButton->setDefault(true);
  
  connect(closeButton, SIGNAL(clicked()), this, SLOT(close()));
  connect(validateButton, SIGNAL(clicked()), this, SLOT(submit()));

  QDialogButtonBox * buttonBox = new QDialogButtonBox;
  buttonBox->addButton(validateButton, QDialogButtonBox::AcceptRole);
  buttonBox->addButton(closeButton, QDialogButtonBox::RejectRole);
  
  // Create main layout
  QVBoxLayout * layout = new QVBoxLayout;
  layout->addWidget(box);
  layout->addWidget(buttonBox);
  setLayout(layout);
}


void WineDialog::fillWine(void)
{
  QSqlQuery query;
  int id;
  // Get selected wine infos
  QSqlRecord record = m_model->record(m_wineIdx);
  id = m_category->findText(
  	record.value(Cellar::WineColumn_Category).toString());
  m_category->setCurrentIndex(id);

  int millesime = record.value(Cellar::WineColumn_Millesime).toInt();
  if (millesime == 0) {
		// Wine with no millesime
		m_millesimeCheckBox->setChecked(true);
		m_millesime->setDisabled(true);
	} else {
		m_millesime->setValue(millesime);
		m_millesimeCheckBox->setChecked(false);
		m_millesime->setDisabled(false);
	}

  id = m_appellation->findText(
	record.value(Cellar::WineColumn_Appellation).toString());
  m_appellation->setCurrentIndex(id);

  m_name->setText(
	record.value(Cellar::WineColumn_Name).toString());
  m_quantity->setValue(
	record.value(Cellar::WineColumn_Quantity).toInt());
  m_boughtQuantity->setValue(
	record.value(Cellar::WineColumn_BoughtQuantity).toInt());
  m_boughtPrice->setText(
	record.value(Cellar::WineColumn_BoughtPrice).toString());

  QDate date = 
	QDate::fromString(record.value(Cellar::WineColumn_BoughtDate).toString(), 
					  "d/M/yyyy");
  m_boughtDate->setDate(date);
  
  id = m_country->findText(
	record.value(Cellar::WineColumn_Country).toString());
  m_country->setCurrentIndex(id);
  
  id = m_productor->findText(
	record.value(Cellar::WineColumn_Productor).toString());
  m_productor->setCurrentIndex(id);

  id = m_volume->findText(
	record.value(Cellar::WineColumn_Volume).toString());
  m_volume->setCurrentIndex(id);

  id = m_place->findText(
	record.value(Cellar::WineColumn_Place).toString());
  m_place->setCurrentIndex(id);

  m_bestBefore->setValue(
	record.value(Cellar::WineColumn_BestBefore).toInt());
  m_comment->setText(
	record.value(Cellar::WineColumn_Comment).toString());
}


void WineDialog::submit(void)
{
  QString name = m_name->text();
  
  if (name.isEmpty()) {
	  QString message(tr("Please provide the name of the wine."));
	  QMessageBox::information(this, tr("Add wine"), message);
  } else {
	QSqlRecord record;
	
	QSqlField f1("name", QVariant::String);
	QSqlField f2("category_id", QVariant::Int);
	QSqlField f3("millesime", QVariant::Int);
	QSqlField f4("appellation_id", QVariant::Int);
	QSqlField f5("quantity", QVariant::Int);
	QSqlField f6("bought_quantity", QVariant::Int);
	QSqlField f7("bought_price", QVariant::Double);
	QSqlField f8("bought_date", QVariant::String);
	QSqlField f9("country_id", QVariant::Int);
	QSqlField f10("productor_id", QVariant::Int);
	QSqlField f11("volume", QVariant::Int);
	QSqlField f12("place_id", QVariant::Int);
	QSqlField f13("best_before", QVariant::Int);
	QSqlField f14("comment", QVariant::String);
	
	f1.setValue(QVariant(name));
	int cat = m_cellar->findCategoryId(m_category->currentText());
	f2.setValue(QVariant(cat));
	int millesime;
	if (m_millesimeCheckBox->isChecked()) {
		millesime = 0;
	} else {
		millesime = m_millesime->value();
	}

	f3.setValue(QVariant(millesime));
	int appel = m_cellar->findAppellationId(m_appellation->currentText());
	f4.setValue(QVariant(appel));
	if (m_wineIdx != -1) {
	  f5.setValue(QVariant(m_quantity->value()));
	} else {
	  f5.setValue(QVariant(m_boughtQuantity->value()));
	}
	f6.setValue(QVariant(m_boughtQuantity->value()));
	f7.setValue(QVariant(m_boughtPrice->text()));
	f8.setValue(QVariant(m_boughtDate->date().toString()));
	int country = m_cellar->findCountryId(m_country->currentText());
	f9.setValue(QVariant(country));
	int prod = m_cellar->findProductorId(m_productor->currentText());
	f10.setValue(QVariant(prod));
	int vol = m_cellar->findVolumeId(m_volume->currentText());
	f11.setValue(QVariant(vol));
	int place = m_cellar->findPlaceId(m_place->currentText());
	f12.setValue(QVariant(place));
	f13.setValue(QVariant(m_bestBefore->value()));
	f14.setValue(QVariant(m_comment->toPlainText()));
	
	record.append(f1);
	record.append(f2);
	record.append(f3);
	record.append(f4);
	record.append(f5);
	record.append(f6);
	record.append(f7);
	record.append(f8);
	record.append(f9);
	record.append(f10);
	record.append(f11);
	record.append(f12);
	record.append(f13);
	record.append(f14);
	
	if (m_wineIdx == -1) {
	  m_model->insertRecord(-1, record);
	} else {
	  m_model->setRecord(m_wineIdx, record);
	}

	m_model->submitAll();
	accept();
  }
}

void WineDialog::onAddAppellation(void)
{
  AppellationDialog * dialog = new AppellationDialog(m_cellar, this);
  if (dialog->exec()) {
	QString n = dialog->appellation();
	int i = m_appellation->findText(n);
	m_appellation->setCurrentIndex(i);
  }
}

void WineDialog::onAddProductor(void)
{
  ProductorDialog * dialog = new ProductorDialog(m_cellar, this);
  if (dialog->exec()) {
	QString n = dialog->productor();
	int i = m_productor->findText(n);
	m_productor->setCurrentIndex(i);
  }
}

void WineDialog::onAddCountry(void)
{
  CountryDialog * dialog = new CountryDialog(m_cellar, this);
  if (dialog->exec()) {
	QString n = dialog->country();
	int i = m_country->findText(n);
	m_country->setCurrentIndex(i);
  }
}


void WineDialog::onAddPlace(void)
{
  PlaceDialog * dialog = new PlaceDialog(m_cellar, this);
  if (dialog->exec()) {
	QString n = dialog->place();
	int i = m_place->findText(n);
	m_place->setCurrentIndex(i);
  }
}

void WineDialog::onAppellationChanged(const QString & app) {
	QSqlRecord rec = m_cellar->findAppellationByName(app);
	if (!rec.isEmpty()) {
		// Find country index from country name 
		QString n = rec.value(Cellar::AppellationColumn_Country).toString();
		int pos = m_country->findText(n);
		m_country->setCurrentIndex(pos);
	}
}

void WineDialog::onMillesimeCheckBoxChanged(int state) {
	m_millesime->setDisabled(state != Qt::Unchecked);
}
